SystemOrganization addCategory: #'Athens-Core'!
SystemOrganization addCategory: 'Athens-Core-Matrices'!
SystemOrganization addCategory: 'Athens-Core-Paints'!
SystemOrganization addCategory: 'Athens-Core-Paths'!
SystemOrganization addCategory: 'Athens-Core-PathsGeometry'!

Object subclass: #AthensAbstractPaint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paints'!

!AthensAbstractPaint commentStamp: 'FernandoOlivero 3/9/2012 14:03' prior: 0!
I am an abstract paint, which should be converted before it can be used by Athens.
I am backend agnostic, as opposed to concrete paint(s).
Also, any other object can play the role of paint, as long as it implements the conversion method, #asAthensPaintOn:

See other implementors of #asAthensPaintOn:, such as Color and Form.!

!AthensAbstractPaint methodsFor: 'converting' stamp: 'IgorStasenko 9/3/2013 14:23'!
asAthensPaintOn: aCanvas	
	^ self! !

AthensAbstractPaint subclass: #AthensPaint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paints'!

!AthensPaint commentStamp: '<historical>' prior: 0!
I am a base class, which defines a paint.

My subclasses represent a concreate paint by implementing my protocol,  that actually draws a specific shapes with me. My subclasses usually backend-specific, but as long as you implement my protocol, any object can be treated as paint:

 - athensFillPath: aPath on: aCanvas
 - athensFillRectangle: aRectangle on: aCanvas
 - asStrokePaint





!

!AthensPaint methodsFor: 'converting' stamp: 'IgorStasenko 4/27/2012 11:28'!
asStrokePaintOn: aCanvas
	"default implementation"
	^ aCanvas surface createStrokePaintFor: self
	! !

!AthensPaint methodsFor: 'rendering' stamp: 'IgorStasenko 9/3/2013 14:22'!
athensFillPath: aPath on: aCanvas 

	"This is a terminal method in rendering dispatch scheme
	canvas->shape->paint. See AthensCanvas>>fillShape: "
	
	self subclassResponsibility! !

!AthensPaint methodsFor: 'rendering' stamp: 'IgorStasenko 9/3/2013 14:23'!
athensFillRectangle: aRectangle on: aCanvas
	"This is a terminal method in rendering dispatch scheme
	canvas->shape->paint. See AthensCanvas>>fillShape: "
	
	self subclassResponsibility! !

AthensPaint subclass: #AthensShadowPaint
	instanceVariableNames: 'color width blur offset'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paints'!

!AthensShadowPaint commentStamp: 'AliakseiSyrel 11/26/2015 19:53' prior: 0!
I representing a shadow paint object used by Athens.
My subclasses provide backend-specific implementation.
I am more serving to define the common protocols and
requirements for all backends (such as defaults).


IMPORTANT NOTE: a shadow paint object provides a protocol 
only for setting shadow properties, but not retrieving them back.
This is intentionally, because the way how these properties is
managed are highly backend-specific. 

The shadow paint has following properties:
 - color. A shadow color defined in RGBA color space.

	Default: Color black alpha: 0.5

 - shadow width. Can be set using #width:

	Default: if width is not set explicitly for shadow paint,
		it is assumed to be equal to 2.0.
		
 - shadow offset. Defines how far shadow is located.
	Shadow offset is a point where x component defines ofset on x-asix and y component on y-axis.
	
	Default: if offset is not set  explicitely it is assumed to be equal 2@2.
			
 - shadow blur. Defines how smooth shadow is.
	Blur is a point with x component defining bluring along x-axis and y component along y axis
	
	Default: if blur is not set explicitely it is assumed to be equal 4@4
!

!AthensShadowPaint methodsFor: 'converting' stamp: 'AliakseiSyrel 11/26/2015 19:54'!
asStrokePaintOn: aCanvas
	"receiver can be converted to stroke paint
	presering color and width"
	^ (aCanvas surface createStrokePaintFor: self color)
		width: self width! !

!AthensShadowPaint methodsFor: 'accessing' stamp: 'AliakseiSyrel 11/26/2015 19:39'!
blur
	^ blur! !

!AthensShadowPaint methodsFor: 'accessing' stamp: 'AliakseiSyrel 11/26/2015 19:41'!
blur: aPoint
	"Set shadpw blur"
	blur := aPoint! !

!AthensShadowPaint methodsFor: 'accessing' stamp: 'AliakseiSyrel 11/26/2015 19:53'!
color
	^ color! !

!AthensShadowPaint methodsFor: 'accessing' stamp: 'AliakseiSyrel 11/26/2015 19:53'!
color: aColor
	color := aColor! !

!AthensShadowPaint methodsFor: 'initialization' stamp: 'AliakseiSyrel 11/26/2015 19:51'!
initialize
	super initialize.
	
	self setDefaults! !

!AthensShadowPaint methodsFor: 'accessing' stamp: 'AliakseiSyrel 11/26/2015 19:39'!
offset
	^ offset! !

!AthensShadowPaint methodsFor: 'accessing' stamp: 'AliakseiSyrel 11/26/2015 19:41'!
offset: aPoint
	"Set shadow offset"
	offset := aPoint! !

!AthensShadowPaint methodsFor: 'initialization' stamp: 'AliakseiSyrel 11/26/2015 19:50'!
setDefaults

	"Set the default values for shadows"
	self
		color: (Color black alpha: 0.5);
		width: 2;
		offset: 2@2; 
		blur: 4@4
	
	
	! !

!AthensShadowPaint methodsFor: 'accessing' stamp: 'AliakseiSyrel 11/26/2015 19:39'!
width
	^ width! !

!AthensShadowPaint methodsFor: 'accessing' stamp: 'AliakseiSyrel 11/26/2015 19:40'!
width: aNumber
	"Set shadow width"
	width := aNumber! !

AthensPaint subclass: #AthensStrokePaint
	instanceVariableNames: 'fillPaint width miterLimit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paints'!

!AthensStrokePaint commentStamp: '<historical>' prior: 0!
I representing a stroke paint object used by Athens.
My subclasses provide backend-specific implementation.
I am more serving to define the common protocols and
requirements for all backends (such as defaults).


IMPORTANT NOTE: a stroke paint object provides a protocol 
only for setting stroke properties, but not retrieving them back.
This is intentionally, because the way how these properties is
managed are highly backend-specific. 

The stroke paint has following properties:
 - fill paint. A paint to use for filling strokes.
	Can be any athens basic paint, except from stroke one.

 - stroke width. Can be set using #width:

	Default: if width is not set explicitly for stroke paint,
		it is assumed to be equal to 1.0.
		
 - join style. 
		Currently there are 3 kinds of joins supported:
			bevel
			miter
			round
		To set join style for paint, use 
			#joinBevel, #joinMiter or #joinRound methods.

	Default: if join style is not explicitly set for stroke paint,
		it will use bevel join style.

 - cap style
		supported: butt, round and square.
		To set cap style, use
			#capButt, #capRound or #capSquare methods.

	Default: if cap style is not explicitly set for stroke paint,
		it will use butt cap style.

 - miter limit, set with #miterLimit: accessor.
	(default and meaning of limit is not yet determined)

Dashes: 
	stroke can use dashing. 
	Dash is special kind of stroke which won't draw a continuous
	stroke connecting path segments, but instead stroke with
	alternating fill-gap style , defined by provided input.
	
	Protocol:
	
	paint 
		dashes: dashPattern 
		offset: anOffset.
		
	The dash pattern is a simple 
	collection of alternating lengths, like: 
	#("fill" 50 "gap" 50) 
	
	in this example , first 50 length units of path
	will be filled with stroke, and next 50 will be skipped,
	forming a gap, then again filled and again skipped,
	and so on until path ends.
	The dash pattern can contain as many length elements
	as needed. Just remember that each odd element represents
	length to fill with stroke, and each even element, length to skip,
	while traversing along the path.
	
	The offset controls the shift in length units, 
	relative to path start.
	
	Default: if dash is not explicitly set for stroke paint,
		it will fill the path using continuous stroke (no gaps). !

!AthensStrokePaint methodsFor: 'converting' stamp: 'IgorStasenko 4/27/2012 11:12'!
asStrokePaintOn: aCanvas
	"receiver is stroke paint already"
	^ self! !

!AthensStrokePaint methodsFor: 'setting cap styles' stamp: 'IgorStasenko 9/3/2013 13:08'!
capButt
	"Set this paint to use butt cap style"
	self subclassResponsibility! !

!AthensStrokePaint methodsFor: 'setting cap styles' stamp: 'IgorStasenko 9/3/2013 13:08'!
capRound
	"Set this paint to use round cap style"
	self subclassResponsibility! !

!AthensStrokePaint methodsFor: 'setting cap styles' stamp: 'IgorStasenko 9/3/2013 13:08'!
capSquare
	"Set this paint to use square cap style"
	self subclassResponsibility! !

!AthensStrokePaint methodsFor: 'setting dashes' stamp: 'IgorStasenko 9/3/2013 13:36'!
dashes: anAlternateCollectionOfLenghts offset: anOffset
	
	'self dashes: #( "fill" 20  "gap" 10 "fill" 35 "gap" 30) offset: 0. '.
	
	self subclassResponsibility

	! !

!AthensStrokePaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/29/2011 15:28'!
fillPaint: anObject

	fillPaint := anObject! !

!AthensStrokePaint methodsFor: 'initialize-release' stamp: 'IgorStasenko 9/3/2013 14:18'!
initialize
	self setDefaults.! !

!AthensStrokePaint methodsFor: 'setting join styles' stamp: 'IgorStasenko 9/3/2013 14:19'!
joinBevel
	"Set receiver to use bevel join style"
	self subclassResponsibility! !

!AthensStrokePaint methodsFor: 'setting join styles' stamp: 'IgorStasenko 9/3/2013 14:19'!
joinMiter
	"Set receiver to use miter join style"
	self subclassResponsibility! !

!AthensStrokePaint methodsFor: 'setting join styles' stamp: 'IgorStasenko 9/3/2013 14:19'!
joinRound
	"Set receiver to use round join style"
	self subclassResponsibility! !

!AthensStrokePaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/29/2011 15:28'!
miterLimit: anObject

	miterLimit := anObject! !

!AthensStrokePaint methodsFor: 'initialize-release' stamp: 'IgorStasenko 9/3/2013 13:13'!
setDefaults

	"Set the default values for strokes"
	width := 1.
	self 
		capButt; 
		joinBevel;
		miterLimit: 4.0
	
	
	! !

!AthensStrokePaint methodsFor: 'accessing' stamp: 'NicolaiHess 1/6/2015 19:35'!
width
	^ width! !

!AthensStrokePaint methodsFor: 'accessing' stamp: 'IgorStasenko 9/3/2013 13:10'!
width: anObject
	"set the stroke width"
	width := anObject! !

AthensAbstractPaint subclass: #GradientPaint
	instanceVariableNames: 'stops colorRamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paints'!

!GradientPaint commentStamp: '<historical>' prior: 0!
I am paint which using color gradient for fills.

The gradient is defined using color ramp in range between 0 and 1
where colors can be set at certain point. The color are interpolated between defined points.

The color ramp is a collection of associations with keys - floating point values between 0 and 1
and values with Colors, for example:

{  0 -> Color white. 1 -> Color black }.

My subclasses used to define a concrete kind of gradient,
while me is an abstract class which has common properties for all gradients: color ramp.

Also, note that my subclasses representing backend-neutral gradients,
encapsulating all necessary data, which is then passed to corresponding surface factory method, like
AthensSurface>>#createLinearGradient:start:stop: , which answers backend-specific gradient paint.
I am userful in cases when user wants to define a paint without need
to have any Athens surface at hand.

Sure thing, using backend-specific gradients is preferable, if possible,
because conversion to backend-specific paint can be costly. 
Especially if gradient is statically defined (ramp values and other parameters never changing) 
and used many times during the life time of surface.

!

!GradientPaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/12/2013 09:54'!
atStop: anOffset put: aColor

	colorRamp add: (anOffset -> aColor).
! !

!GradientPaint methodsFor: 'accessing' stamp: 'FernandoOlivero 1/13/2012 21:11'!
colorRamp
	^ colorRamp! !

!GradientPaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/13/2013 09:11'!
colorRamp: aRamp
"The color ramp is a collection of associations with keys - floating point values between 0 and 1
and values with Colors, for example:

{  0 -> Color white. 1 -> Color black }.
"
	colorRamp := aRamp copy.! !

!GradientPaint methodsFor: 'initialize-release' stamp: 'IgorStasenko 4/12/2013 09:51'!
initialize
	super initialize.
	colorRamp := OrderedCollection new.! !

GradientPaint subclass: #LinearGradientPaint
	instanceVariableNames: 'start stop'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paints'!

!LinearGradientPaint commentStamp: '<historical>' prior: 0!
I represent a linear gradient paint.
My start and stop points defining 
the position and direction of gradient fill 
in user space coordinates,
so that the ramp color at 0 maps to starting point
and ramp color at 1 maps to stop point of linear gradient.



Also check the comment of my superclass to know better how and when you can use me.
!

!LinearGradientPaint class methodsFor: 'as yet unclassified' stamp: 'maxmattone 6/12/2015 10:27'!
from: aStartPoint  to: aStopPoint
	^ self new start: aStartPoint; stop: aStopPoint; yourself.! !

!LinearGradientPaint methodsFor: 'converting' stamp: 'IgorStasenko 4/12/2013 10:14'!
asAthensPaintOn: aCanvas
	^ aCanvas surface
			createLinearGradient: self colorRamp
				start: self start 
				stop: self stop! !

!LinearGradientPaint methodsFor: 'accessing' stamp: 'FernandoOlivero 1/13/2012 23:10'!
start

	^ start! !

!LinearGradientPaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/12/2013 09:56'!
start: anObject
	
	start := anObject! !

!LinearGradientPaint methodsFor: 'accessing' stamp: 'FernandoOlivero 1/13/2012 23:10'!
stop

	^ stop! !

!LinearGradientPaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/12/2013 09:56'!
stop: anObject
	
	stop := anObject! !

GradientPaint subclass: #RadialGradientPaint
	instanceVariableNames: 'center radius focalPoint'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paints'!

!RadialGradientPaint commentStamp: '<historical>' prior: 0!
I represent a radial gradient paint.

The center point and radius defining a circle where gradient will be drawn in user space coordinates.

The gradient's ramp color at 0 maps to the focal point and ramp color at 1 maps to the edge of the circle,
interpolating colors in between.

Also check the comment of my superclass to know better how and when you can use me.
!

!RadialGradientPaint methodsFor: 'converting' stamp: 'IgorStasenko 4/12/2013 09:33'!
asAthensPaintOn: aCanvas
	^ aCanvas surface
		createRadialGradient:  colorRamp
		center: center
		radius: radius
		focalPoint: focalPoint ! !

!RadialGradientPaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/12/2013 09:26'!
center
	
	^ center! !

!RadialGradientPaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/12/2013 09:22'!
center: aPoint

	center := aPoint! !

!RadialGradientPaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/12/2013 09:26'!
focalPoint
	
	^ focalPoint! !

!RadialGradientPaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/12/2013 09:24'!
focalPoint: aPoint

	focalPoint := aPoint! !

!RadialGradientPaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/12/2013 09:26'!
radius
	
	^ radius! !

!RadialGradientPaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/12/2013 09:23'!
radius: aNumber

	radius := aNumber! !

Object subclass: #AthensAbstractShape
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core'!

!AthensAbstractShape commentStamp: '<historical>' prior: 0!
In Athens, any object can play role of a shape.

I demonstrating the minimal protocol which should be supported
by "shape" role, to be used with Athens canvas.
See my methods comments for more details.
!

!AthensAbstractShape methodsFor: 'converting' stamp: 'IgorStasenko 5/6/2013 05:40'!
asAthensShapeOn: canvas
	"Note: The answered object of this message should conform with AthensShape protocol.	"
	self subclassResponsibility.! !

AthensAbstractShape subclass: #AthensShape
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core'!

!AthensShape commentStamp: '<historical>' prior: 0!
This class represents an interface, which should be supported by all shapes used for drawing with Athens framework.

Shapes do not have to inherit directly from me, for example Rectangle implements my protocol (via extensions), and as result can be used as a valid shape for drawing with Athens.!

AthensShape subclass: #AthensPath
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core'!

!AthensPath commentStamp: '<historical>' prior: 0!
This class represents an interface, which should be supported by all paths (special kind of shape) used for drawing with Athens framework.

Path data organization is up to surface where it built,
therefore paths are not interchangeable between different surfaces.

!

!AthensShape methodsFor: 'converting' stamp: 'IgorStasenko 5/6/2013 05:32'!
asAthensShapeOn: canvas
	"Receiver is a shape, no conversion is needed."
	^ self! !

!AthensShape methodsFor: 'drawing' stamp: 'IgorStasenko 12/20/2011 18:36'!
paintFillsUsing: aPaint on: anAthensCanvas 
	"This method is a part of rendering dispatch  Canvas->receiver->paint"
	
	self subclassResponsibility! !

Object subclass: #AthensBezier3Scene
	instanceVariableNames: 'pt1 pt2 pt3 pt4'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-PathsGeometry'!

!AthensBezier3Scene commentStamp: '<historical>' prior: 0!
I just a helper class used to debug & visualize rendering of bezier segment. This class is subject of changes or complete removal!

!AthensBezier3Scene class methodsFor: 'instance creation' stamp: 'IgorStasenko 4/26/2013 17:51'!
on: anControlPointArray

	^ self basicNew
		initFromArray: anControlPointArray! !

!AthensBezier3Scene methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/29/2013 21:51'!
curvePathBlock
	^ [ :builder |
		builder 
			absolute;
			moveTo: pt1;
			curveVia: pt2 and: pt3 to: pt4
	 ].
! !

!AthensBezier3Scene methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/26/2013 17:54'!
initFromArray: aControlPointsArray
	
		pt1 := aControlPointsArray at: 1.
		pt2 := aControlPointsArray at: 2.
		pt3 := aControlPointsArray at: 3.
		pt4 := aControlPointsArray at: 4.
! !

!AthensBezier3Scene methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 5/2/2013 18:25'!
renderOn: canvas

	| path pathBlock polygonizedPath |
	
	canvas surface clear:  Color black.
	pathBlock :=  [ :builder |
		builder 
			absolute;
			moveTo: pt1;
			curveVia: pt2 and: pt3 to: pt4
	 ].
	

	"First , we draw the curve using Cairo"	
	(canvas setStrokePaint: Color green).
	canvas drawShape: (canvas createPath: pathBlock).
		
	"draw a polygon, connecting control points"
	(canvas setStrokePaint: Color blue) width: 0.5.
	canvas drawShape: (canvas createPath: [ :builder |
		builder 
			absolute;
			moveTo: pt1;
			lineTo: pt2;
			lineTo: pt3;
			lineTo: pt4
		 ]).
	
		
	
	canvas setPaint: Color red.
	
	path := AthensSimplePathBuilder createPath: pathBlock.
"	polygonizedPath := path asPolygon: canvas pathTransform. "
	
	canvas drawShape: (
		canvas createPath: [ :builder | | flattener |
			builder absolute.
	
		flattener := AthensCurveFlattener new dest: builder.
		flattener transform: (canvas pathTransform). 
		flattener convert:path. 
	]).! !

Object subclass: #AthensCanvas
	instanceVariableNames: 'paint surface shape fontRenderer paintOptions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core'!

!AthensCanvas commentStamp: '<historical>' prior: 0!
The Athens canvas is a central object which is used to performs drawings on a surface.
Please note, that Athens does not allows you to instantiate canvas directly, instead you obtain a ready for use instance as an argument in 
#drawDuring: message, sent to athens surface:

surface drawDuring: [:canvas | .... ]

Using canvas outside a #drawDuring: method is highly discouraged. Doing so may lead to unpredicted behavior/data corruption/image crash.!

!AthensCanvas class methodsFor: 'private' stamp: 'IgorStasenko 3/21/2011 00:19'!
new
	"use #on: instead"
	self shouldNotImplement! !

!AthensCanvas class methodsFor: 'private' stamp: 'IgorStasenko 3/21/2011 00:17'!
on: aSurface
	^ self basicNew initializeWithSurface: aSurface! !

!AthensCanvas methodsFor: 'caching' stamp: 'IgorStasenko 10/12/2012 03:20'!
cacheAt: anObject ifAbsentPut: aBlock

	^ surface cacheAt: anObject ifAbsentPut: aBlock! !

!AthensCanvas methodsFor: 'masking' stamp: 'IgorStasenko 3/27/2011 18:31'!
clearMask: aShape
	"set mask pixels to 0 for all pixels covered by shape, aShape"
	^ aShape paintFillsUsing: surface clearMaskPaint on: self
! !

!AthensCanvas methodsFor: 'clipping' stamp: 'SeanDeNigris 4/27/2015 10:59'!
clipBy: aRectangle during: aBlock
	
	"Set a clipping rectangle during drawing operations performed in aBlock.
	Note that the clipping rectangle is intersected with the currently active clipping rectangle"
	
	 ^surface clipBy: aRectangle during: aBlock! !

!AthensCanvas methodsFor: 'paths' stamp: 'IgorStasenko 11/19/2012 05:21'!
createPath: aPathCreatingBlock
	"Create new path. A path creating block should be a monadic block,
	which will get a path builder instance as argument. See AthensPathBuilder
	for available protocol.
	The resulting path object can be later used for drawing.
	" 
	^ surface createPath: aPathCreatingBlock! !

!AthensCanvas methodsFor: 'paths' stamp: 'IgorStasenko 3/22/2011 22:04'!
createRectanglePath: aRectangle 
	^ surface createRectanglePath: aRectangle 
! !

!AthensCanvas methodsFor: 'masking' stamp: 'IgorStasenko 3/27/2011 18:06'!
disableMask
	^ surface disableMask! !

!AthensCanvas methodsFor: 'drawing' stamp: 'IgorStasenko 7/3/2013 16:28'!
draw
	"Fill the currently selected shape with currently selected paint"
	
	^ shape paintFillsUsing: paint on: self

! !

!AthensCanvas methodsFor: 'drawing' stamp: 'IgorStasenko 11/19/2012 05:18'!
draw: anObject
	"A generic dispatch method to draw anObject on receiver. 
	the object should understand #drawOnAthensCanvas: message"
	^ anObject drawOnAthensCanvas: self! !

!AthensCanvas methodsFor: 'drawing' stamp: 'IgorStasenko 11/19/2012 05:17'!
drawShape: anObject
	"A convenience method, which sets the current shape to anObject and then fills it
	with currently selected paint" 
	self setShape: anObject.
	self draw.
	! !

!AthensCanvas methodsFor: 'drawing text' stamp: 'IgorStasenko 9/1/2012 22:08'!
drawString: aString
	"Draw a string using currently selected font.
	Answer a total advance of rendered string "
	^ fontRenderer renderCharacters: aString from: 1 to: aString size.! !

!AthensCanvas methodsFor: 'drawing text' stamp: 'IgorStasenko 9/1/2012 22:08'!
drawString: aString from: start to: end

	"Draw a portion of string using currently selected font.
	Answer a total advance of rendered portion"
	^ fontRenderer renderCharacters: aString from: start to: end! !

!AthensCanvas methodsFor: 'masking' stamp: 'IgorStasenko 3/27/2011 18:06'!
enableMask
	^ surface enableMask! !

!AthensCanvas methodsFor: 'caching' stamp: 'AlainPlantec 3/3/2015 18:41'!
flushCacheAt: anObject
	"Flush (delete) any cached value(s) identified by given object, anObject.
	A surface using identity comparison for object identifiers.

	Answer receiver.
	"

	surface flushCacheAt: anObject! !

!AthensCanvas methodsFor: 'initialization' stamp: 'IgorStasenko 3/22/2011 11:13'!
initializeWithSurface: anAthensSurface	
	surface := anAthensSurface! !

!AthensCanvas methodsFor: 'clipping' stamp: 'IgorStasenko 4/21/2011 14:40'!
isVisible: aRectangle 
	"check if rectangle are not fully clipped by current clipping coordinates"
	
	^ true! !

!AthensCanvas methodsFor: 'accessing' stamp: 'FernandoOlivero 2/14/2012 12:12'!
paint
	^ paint  ! !

!AthensCanvas methodsFor: 'accessing' stamp: 'AliakseiSyrel 6/4/2016 01:17'!
paintMode
	"Answer the current paint mode controller.
	The answered object should conform to AthensPaintMode public protocol."
	self deprecated: 'Use paintOptions instead'
		on: 	'4 June 2016' 
		in: 'Athens-Core'.
	^ self paintOptions! !

!AthensCanvas methodsFor: 'accessing' stamp: 'AliakseiSyrel 6/4/2016 01:15'!
paintOptions
	"Answer the current paint options controller.
	The answered object should conform to AthensPaintOptions public protocol."

	^ paintOptions! !

!AthensCanvas methodsFor: 'transformation matrices' stamp: 'IgorStasenko 11/19/2012 05:14'!
paintTransform
	"Answer the paint transformation matrix controller.
	The answered object should conform to AthensTransform public protocol."

	^ surface paintTransform
! !

!AthensCanvas methodsFor: 'transformation matrices' stamp: 'IgorStasenko 11/19/2012 05:14'!
pathTransform
	"Answer the path transformation matrix controller.
	The answered object should conform to AthensTransform public protocol."
	^ surface pathTransform
! !

!AthensCanvas methodsFor: 'drawing text' stamp: 'IgorStasenko 8/30/2013 16:14'!
setFont: aFont
	"Set the current font of receiver. Font object should answer a glyph renderer instance,
	compatible with glyph renderer protocol" 
	^ fontRenderer := aFont glyphRendererOn: surface.! !

!AthensCanvas methodsFor: 'masking' stamp: 'IgorStasenko 3/27/2011 18:35'!
setMask: aShape
	"set mask pixels to 1 for all pixels covered by shape, aShape"

	^ aShape paintFillsUsing: surface setMaskPaint on: self
! !

!AthensCanvas methodsFor: 'paint' stamp: 'IgorStasenko 11/19/2012 05:11'!
setPaint: aPaint
	"set the current paint of receiver"
	paint = aPaint ifTrue:[ ^ paint ].
	paint := aPaint asAthensPaintOn: self.
	^ paint.! !

!AthensCanvas methodsFor: 'accessing' stamp: 'IgorStasenko 5/6/2013 05:41'!
setShape: anObject
	"Set the current shape of receiver"
	shape := anObject asAthensShapeOn: self.! !

!AthensCanvas methodsFor: 'paint' stamp: 'IgorStasenko 11/19/2012 05:11'!
setStrokePaint: aPaint
	"use paint as stroke paint, note conversion to #asStrokePaintOn:"
	paint := (aPaint asAthensPaintOn: self) asStrokePaintOn: self.
	^ paint! !

!AthensCanvas methodsFor: 'accessing' stamp: 'IgorStasenko 11/19/2012 05:10'!
surface
	"Answer an Athens surface, to which receiver is bound to"
	^ surface! !

Object subclass: #AthensCubicBezier
	instanceVariableNames: 'x1 y1 x2 y2 x3 y3 x4 y4'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-PathsGeometry'!

!AthensCubicBezier commentStamp: '<historical>' prior: 0!
Cubic bezier with 4 control points encapsulation.

Main purpose of this class is to keep subdivision logic in clean and separate place!

!AthensCubicBezier methodsFor: 'subdivision' stamp: 'IgorStasenko 4/29/2013 21:42'!
recursiveSubDiv: flattener level: level

	| dx dy d2 d3 da1 da2 k f1 f2 |
	
	
	" Try to approximate the full cubic curve by a single straight line "
	level > flattener subdivisionLimit ifTrue: [  ^ self ].
	
	dx := x4-x1.
	dy := y4-y1.

	d2 := ((x2 - x4) * dy - ((y2 - y4) * dx)) abs.
	d3 := ((x3 - x4) * dy - ((y3 - y4) * dx)) abs.

	f1 := d2 > flattener curveCollinearityEpsilon.
	f2 := d3 > flattener curveCollinearityEpsilon.
	

	f1 ifTrue: [ 
		f2 ifTrue: [
			" Regular case "
			
			(d2 + d3) squared <= (flattener distanceToleranceSquared * (dx squared + dy squared)) ifTrue: [ 
				"If the curvature doesn't exceed the distance_tolerance value
				we tend to finish subdivisions."
			"	flattener accountForAngleTolerance ifFalse: [
					^ flattener lineToX: (x2 interpolateTo: x3 at: 0.5) y: (y2 interpolateTo: y3 at: 0.5)
					].
			"
				"Angle & Cusp Condition"

				k := (y3-y2) arcTan: (x3 - x2).
				da1 := (k - ((y2-y1) arcTan: (x2-x1))) abs.
				da2 := ((y4-y3 arcTan: x4-x3) - k) abs.

				da1 >= Float pi ifTrue: [  da1 := Float pi*2 - da1 ].
				da2 >= Float pi ifTrue: [  da2 := Float pi*2 - da2 ].
			
				(da1 + da2) < flattener angleTolerance ifTrue: [ 
					" Finally we can stop the recursion "
					^ flattener lineToX: (x2 interpolateTo: x3 at: 0.5) y: (y2 interpolateTo: y3 at: 0.5)
					 ].
		 
				(flattener overCuspLimit: da1) ifTrue: [ ^ flattener lineToX: x2 y: y2 ].
            	   	(flattener overCuspLimit: da2) ifTrue: [ ^ flattener lineToX: x3 y: y3 ].
			]
			
		] ifFalse: [ 
			
			"p1,p3,p4 are collinear, p2 is significant"

			(d2 squared <= (flattener distanceToleranceSquared * (dx squared + dy squared) )  )
				ifTrue: [
					flattener accountForAngleTolerance ifFalse: [ 
						^ flattener lineToX: (x2 interpolateTo: x3 at: 0.5)  y: (y2 interpolateTo: y3 at: 0.5) ].

					"Angle Condition"
					da1 := ((y3-y2 arcTan: (x3-x2)) - (y2-y1 arcTan:(x2-x1))) abs.
					da1 >= Float pi ifTrue: [  da1 := Float pi * 2 - da1 ].
					
					da1 < flattener angleTolerance ifTrue: [ 
						^ flattener 
							lineToX: x2 y: y2;
							lineToX: x3 y: y3 ].
						
					(flattener overCuspLimit: da1) ifTrue: [ ^ flattener lineToX: x2 y: y2 ]
				]			
			 ]
	] ifFalse: [
		f2 ifTrue: [
		"p1,p2,p4 are collinear, p3 is significant "
			
			(d3 squared <= (flattener distanceToleranceSquared * (dx squared + dy squared) )  )
				ifTrue: [
					flattener accountForAngleTolerance ifFalse: [ 
						^ flattener lineToX: (x2 interpolateTo: x3 at: 0.5)  y: (y2 interpolateTo: y3 at: 0.5) ].

					"Angle Condition"
					da1 := ((y4-y3 arcTan: (x4-x3)) - (y3-y2 arcTan:(x3-x2))) abs.
					da1 >= Float pi ifTrue: [  da1 := Float pi * 2 - da1 ].
					
					da1 < flattener angleTolerance ifTrue: [ 
						^ flattener 
							lineToX: x2 y: y2;
							lineToX: x3 y: y3 ].
						
					(flattener overCuspLimit: da1) ifTrue: [ ^ flattener lineToX: x3 y: y3 ]
				]			
			  ] ifFalse: [
		"All collinear OR p1==p4 "
			k := dx*dx + (dy*dy).
			
			(k = 0.0) ifTrue: [
				d2 := (x1-x2) squared + (y1-y2) squared.
				d3 := (x3-x4) squared + (y3-y4) squared ] 
			ifFalse: [ 

				k := 1 / k.
				da1 := x2 - x1.
				da2 := y2 - y1.
				d2  := k * (da1*dx + (da2*dy)).
				da1 := x3 - x1.
				da2 := y3 - y1.
				d3  := k * (da1*dx + (da2*dy)).
				
				(d2 > 0.0 and: [ d2 < 1.0 and: [d3>0.0 and: [d3 < 1.0]]]) ifTrue: [ 
						" Simple collinear case, 1---2---3---4  We can leave just two endpoints"
							^ self ].
				d2 <= 0.0 
					ifTrue: [ d2 := (x1-x2) squared + (y1-y2) squared ]
					ifFalse: [  
						d2 >= 1.0 
							ifTrue: [ d2 := (x2-x4) squared + (y2-y4) squared ]
							ifFalse: [ d2 := (x2 - x1 - (d2*dx)) squared + (y2 - y1 - (d2*dy)) squared ]].

				d3 <= 0.0 
					ifTrue: [  d3 := (x3-x1) squared + (y3-y1) squared ]
					ifFalse: [ 
						d3 >= 1.0 
							ifTrue: [ d3 := (x3-x4) squared + (y3-y4) squared   ]
							ifFalse: [ d3 := (x3 - x1 - (d3*dx)) squared + (y3-y1- (d3*dy)) squared ]].
			].

			(d2 > d3) ifTrue: [
				(d2 < flattener distanceToleranceSquared)
					ifTrue: [ ^ flattener lineToX: x2 y: y2 ]
				]
			ifFalse: [ 
				(d3 < flattener distanceToleranceSquared )
					ifTrue: [  ^ flattener lineToX: x3 y: y3 ]
				 ].
		]
	].
	self subdivideAt: 0.5 do:  [  :b1 :b2 |
		b1 recursiveSubDiv: flattener level: level +1.	
		b2 recursiveSubDiv: flattener level: level +1.	
	].
	! !

!AthensCubicBezier methodsFor: 'subdivision' stamp: 'IgorStasenko 4/24/2013 16:21'!
subdivideAt: t do: aBinaryBlock
	| x12 y12 x23 y23 x34 y34 x1223 y1223 x2334 y2334 xsplit ysplit |
	
	x12 := x1 interpolateTo: x2 at: t.
	y12 := y1 interpolateTo: y2 at: t.
	x23 := x2 interpolateTo: x3 at: t.
	y23 := y2 interpolateTo: y3 at: t.
	x34 := x3 interpolateTo: x4 at: t.
	y34 := y3 interpolateTo: y4 at: t.

	x1223 := x12 interpolateTo: x23 at: t.
	x2334 := x23 interpolateTo: x34 at: t.
	y1223 := y12 interpolateTo: y23 at: t.
	y2334 := y23 interpolateTo: y34 at: t.
	
	xsplit := x1223 interpolateTo: x2334 at: t.
	ysplit := y1223 interpolateTo: y2334 at: t.
	
	^ aBinaryBlock 
		value: 
		"p1 - p12 - p1223 - psplit"
		(self copy 
			x2: x12;
			y2: y12;
			x3: x1223;
			y3: y1223;
			x4: xsplit;
			y4: ysplit )
		value:
		"psplit - p2334 - p34 - p4" 
		(self copy 
			x1: xsplit;
			y1: ysplit;
			x2: x2334;
			y2: y2334;
			x3: x34;
			y3: y34
		)	
! !

!AthensCubicBezier methodsFor: 'accessing' stamp: 'IgorStasenko 4/26/2013 17:07'!
x1
	
	^ x1! !

!AthensCubicBezier methodsFor: 'accessing' stamp: 'IgorStasenko 4/26/2013 17:07'!
x1: anObject
	
	x1 := anObject! !

!AthensCubicBezier methodsFor: 'accessing' stamp: 'IgorStasenko 4/26/2013 17:07'!
x2
	
	^ x2! !

!AthensCubicBezier methodsFor: 'accessing' stamp: 'IgorStasenko 4/26/2013 17:07'!
x2: anObject
	
	x2 := anObject! !

!AthensCubicBezier methodsFor: 'accessing' stamp: 'IgorStasenko 4/26/2013 17:07'!
x3
	
	^ x3! !

!AthensCubicBezier methodsFor: 'accessing' stamp: 'IgorStasenko 4/26/2013 17:07'!
x3: anObject
	
	x3 := anObject! !

!AthensCubicBezier methodsFor: 'accessing' stamp: 'IgorStasenko 4/26/2013 17:07'!
x4
	
	^ x4! !

!AthensCubicBezier methodsFor: 'accessing' stamp: 'IgorStasenko 4/26/2013 17:07'!
x4: anObject
	
	x4 := anObject! !

!AthensCubicBezier methodsFor: 'accessing' stamp: 'IgorStasenko 4/26/2013 17:07'!
y1
	
	^ y1! !

!AthensCubicBezier methodsFor: 'accessing' stamp: 'IgorStasenko 4/26/2013 17:07'!
y1: anObject
	
	y1 := anObject! !

!AthensCubicBezier methodsFor: 'accessing' stamp: 'IgorStasenko 4/26/2013 17:07'!
y2
	
	^ y2! !

!AthensCubicBezier methodsFor: 'accessing' stamp: 'IgorStasenko 4/26/2013 17:07'!
y2: anObject
	
	y2 := anObject! !

!AthensCubicBezier methodsFor: 'accessing' stamp: 'IgorStasenko 4/26/2013 17:07'!
y3
	
	^ y3! !

!AthensCubicBezier methodsFor: 'accessing' stamp: 'IgorStasenko 4/26/2013 17:07'!
y3: anObject
	
	y3 := anObject! !

!AthensCubicBezier methodsFor: 'accessing' stamp: 'IgorStasenko 4/26/2013 17:07'!
y4
	
	^ y4! !

!AthensCubicBezier methodsFor: 'accessing' stamp: 'IgorStasenko 4/26/2013 17:07'!
y4: anObject
	
	y4 := anObject! !

Object subclass: #AthensPaintMode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core'!

!AthensPaintMode commentStamp: 'AliakseiSyrel 6/4/2016 01:09' prior: 0!
!!  DEPRECATED
!!!! Use AthensPaintOptions instead.

A paint mode controls how the incoming color (source) will be transferred to surface (destination).

To set a paint mode for canvas, send a message with corresponding mode name to my instance.
My instance(s) is available via canvas protocol, i.e. 'canvas paintMode'.

For example, to set an 'over' paint mode, use following:

	canvas paintMode over.

If you want to restore original paint mode after performing drawing operations, which may change it, use #restoreAfter: method , i.e. 

	canvas paintMode restoreAfter: [
		.. perform any drawing operations here..
		].

Since different backends may support different set of paint modes, the default implementation in AthensPaintMode for 
all mode setters is to signal an error.
To query a set of available paint modes, supported by backend, use #availableModes message.

!

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 17:05'!
add
	self notAvailable! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 17:04'!
atop
	"Set 'atop' painting mode "
	
	self notAvailable! !

!AthensPaintMode methodsFor: 'capabilities' stamp: 'IgorStasenko 9/1/2012 17:01'!
availableModes

	"Answer a collection of paint mode names, currently supported by backend. 
	Different backends may support different sets of paint modes.
	
	You may need to structure your code depending on modes available"
	
	^ #()! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 17:02'!
clear
	"Set 'clear' drawing mode.
	
	Drawing under this mode will clear the surface under the shape boundaries"
	
	self notAvailable! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 17:06'!
colorBurn
	self notAvailable! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 17:06'!
colorDodge
	self notAvailable! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 17:06'!
darken
	self notAvailable! !

!AthensPaintMode methodsFor: 'default mode' stamp: 'IgorStasenko 9/1/2012 16:41'!
default
	"Set the default paint mode. The default paint mode in Athens, is 'over' "
	^ self over! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 17:02'!
dest
	"Set 'dest' drawing mode.
	
	Drawing under this mode will ignore the source, as if nothing drawn"
	
	self notAvailable! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 17:05'!
destAtop
	
	self notAvailable! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 17:05'!
destIn
	
	self notAvailable! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 17:05'!
destOut
	
	self notAvailable! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 17:04'!
destOver
	
	self notAvailable! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 17:07'!
difference
	self notAvailable! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 17:07'!
exclusion
	self notAvailable! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 17:06'!
hardLight
	self notAvailable! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 17:07'!
hslColor
	self notAvailable! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 17:07'!
hslHue
	self notAvailable! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 17:07'!
hslLuminosity
	self notAvailable! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 17:07'!
hslSaturation
	self notAvailable! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 16:48'!
in
	self notAvailable! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 17:06'!
lighten
	self notAvailable! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 17:05'!
multiply
	self notAvailable! !

!AthensPaintMode methodsFor: 'errors' stamp: 'IgorStasenko 9/1/2012 16:49'!
notAvailable
	^ self error: 'A requested paint mode is not supported by backend'! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 17:04'!
out
	"Set 'out' painting mode "
	
	self notAvailable! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 16:50'!
over
	"Set 'over' drawing mode. This is default paint mode.
	
	Drawing under this mode will blend source with destination color using source alpha component"
	
	self notAvailable! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 17:06'!
overlay
	self notAvailable! !

!AthensPaintMode methodsFor: 'convenience' stamp: 'IgorStasenko 9/1/2012 14:40'!
restoreAfter: aBlock

	"restore current paint mode after evaluating a block"
	
	self subclassResponsibility! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 17:05'!
saturate
	self notAvailable! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 17:06'!
screen
	self notAvailable! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 17:06'!
softLight
	self notAvailable! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 16:50'!
source
	"Set 'source' drawing mode.
	
	Drawing under this mode will replace the content with incoming source under the shape boundaries"
	
	self notAvailable! !

!AthensPaintMode methodsFor: 'modes' stamp: 'IgorStasenko 9/1/2012 17:05'!
xor
	self notAvailable! !

Object subclass: #AthensPaintOptions
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core'!

!AthensPaintOptions commentStamp: 'AliakseiSyrel 6/4/2016 00:57' prior: 0!
A paint options control how the incoming color (source) will be transferred to surface (destination).

To set a paint composition operator for canvas, send a message with corresponding operator name to my instance.
My instance(s) is available via canvas protocol, i.e. 'canvas paintOptions'.

For example, to set an 'over' composition operator, use following:

	canvas paintOptions composeOver.

If you want to restore original paint options after performing drawing operations, which may change it, use #restoreAfter: method , i.e. 

	canvas paintOptions restoreAfter: [
		.. perform any drawing operations here..
		].

Since different backends may support different set of paint options, the default implementation in AthensPaintOptions for all option setters is to signal an error.
To query a set of available paint options, supported by backend, use #availableCompositionOperators message.

!

!AthensPaintOptions methodsFor: 'accessing' stamp: 'AliakseiSyrel 6/4/2016 01:13'!
alpha: aFloat

	self notAvailable! !

!AthensPaintOptions methodsFor: 'antialias modes' stamp: 'AliakseiSyrel 6/4/2016 01:11'!
antialiasDefault

	self notAvailable! !

!AthensPaintOptions methodsFor: 'antialias modes' stamp: 'AliakseiSyrel 6/4/2016 01:11'!
antialiasGray

	self notAvailable! !

!AthensPaintOptions methodsFor: 'antialias modes' stamp: 'AliakseiSyrel 6/4/2016 01:11'!
antialiasNone

	self notAvailable! !

!AthensPaintOptions methodsFor: 'antialias modes' stamp: 'AliakseiSyrel 6/4/2016 01:11'!
antialiasSubpixel

	self notAvailable! !

!AthensPaintOptions methodsFor: 'capabilities' stamp: 'AliakseiSyrel 6/4/2016 01:14'!
availableAlpha
	"Answer wether alpha option is supported by backend"

	^ false! !

!AthensPaintOptions methodsFor: 'capabilities' stamp: 'AliakseiSyrel 6/4/2016 01:14'!
availableAntialiasModes
	"Answer a collection of antialias mode names, currently supported by backend. 
	Different backends may support different sets of antialias modes.

	You may need to structure your code depending on modes available"

	^ #()! !

!AthensPaintOptions methodsFor: 'capabilities' stamp: 'AliakseiSyrel 6/4/2016 01:14'!
availableCompositionOperators
	"Answer a collection of composition operator names, currently supported by backend. 
	Different backends may support different sets of composition operators.
	
	You may need to structure your code depending on operators available"

	^ #()! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 00:57'!
composeAdd
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 00:57'!
composeAtop
	"Set 'atop' painting mode "
	
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 00:58'!
composeClear
	"Set 'clear' composition operator.
	
	Drawing under this mode will clear the surface under the shape boundaries"
	
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 00:58'!
composeColorBurn
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 00:58'!
composeColorDodge
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 01:01'!
composeDarken
	self notAvailable! !

!AthensPaintOptions methodsFor: 'default mode' stamp: 'AliakseiSyrel 6/4/2016 01:04'!
composeDefault
	"Set the default composition operator.
	The default composition operator in Athens, is 'over'"
	^ self composeOver! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 01:01'!
composeDest
	"Set 'dest' drawing mode.
	
	Drawing under this mode will ignore the source, as if nothing drawn"
	
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 01:01'!
composeDestAtop
	
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 01:01'!
composeDestIn
	
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 01:01'!
composeDestOut
	
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 01:01'!
composeDestOver
	
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 01:00'!
composeDifference
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 01:00'!
composeExclusion
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 01:00'!
composeHardLight
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 01:00'!
composeHslColor
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 01:00'!
composeHslHue
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 01:00'!
composeHslLuminosity
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 01:00'!
composeHslSaturation
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 01:00'!
composeIn
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 01:00'!
composeLighten
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 00:59'!
composeMultiply
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 00:59'!
composeOut
	"Set 'out' painting mode "
	
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 00:59'!
composeOver
	"Set 'over' drawing mode. This is default paint mode.
	
	Drawing under this mode will blend source with destination color using source alpha component"
	
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 00:59'!
composeOverlay
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 00:59'!
composeSaturate
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 00:59'!
composeScreen
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 00:59'!
composeSoftLight
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 00:59'!
composeSource
	"Set 'source' drawing mode.
	
	Drawing under this mode will replace the content with incoming source under the shape boundaries"
	
	self notAvailable! !

!AthensPaintOptions methodsFor: 'composition operators' stamp: 'AliakseiSyrel 6/4/2016 00:59'!
composeXor
	self notAvailable! !

!AthensPaintOptions methodsFor: 'errors' stamp: 'AliakseiSyrel 6/4/2016 01:03'!
notAvailable
	^ self error: 'A requested paint option is not supported by backend'! !

!AthensPaintOptions methodsFor: 'convenience' stamp: 'AliakseiSyrel 6/4/2016 00:55'!
restoreAfter: aBlock

	"restore current paint options after evaluating a block"
	self subclassResponsibility! !

Object subclass: #AthensPathBuilder
	instanceVariableNames: ''
	classVariableNames: 'ZeroPoint'
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensPathBuilder commentStamp: '<historical>' prior: 0!
i am abstract path builder, to specify an interface protocol which is available for building paths,
by using  canvas or surface #createPath: method.!

!AthensPathBuilder class methodsFor: 'instance creation' stamp: 'IgorStasenko 4/18/2013 17:45'!
createPath: aBlock

	^ self new createPath: aBlock! !

!AthensPathBuilder class methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 3/7/2012 15:26'!
initialize
	ZeroPoint := 0@0! !

AthensPathBuilder initialize!

!AthensPathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/7/2012 14:26'!
absolute
	"next commands will use absolute coordinates for all segment points"
	self subclassResponsibility! !

!AthensPathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/7/2012 14:28'!
ccwArcTo: endPt angle: rot
	" Add a counter-clockwise arc segment, starting from current path endpoint and
	ending at andPt. Angle should be specified in radians "
	

	self subclassResponsibility! !

!AthensPathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/7/2012 14:30'!
close
	"close the current path controur"
	self subclassResponsibility ! !

!AthensPathBuilder methodsFor: 'creating path' stamp: 'IgorStasenko 4/18/2013 17:46'!
createPath:	aBlock

		"aBlock value: self ...."	
	self subclassResponsibility ! !

!AthensPathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/7/2012 14:30'!
curveVia: cp1 and: cp2 to: aPoint
	"Add a cubic bezier curve,
	starting from current path endpoint, using control points cp1, cp2
	and ending at aPoint "
	
	self subclassResponsibility 	! !

!AthensPathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/7/2012 14:29'!
curveVia: cp1 to: aPoint

	"Add a quadric bezier curve,
	starting from current path endpoint, using control point cp1, 
	and ending at aPoint "
	
	self subclassResponsibility ! !

!AthensPathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/7/2012 14:28'!
cwArcTo: endPt angle: rot
	" Add a clockwise arc segment, starting from current path endpoint and
	ending at andPt. Angle should be specified in radians "
	

	self subclassResponsibility! !

!AthensPathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/7/2012 14:27'!
lineTo: aPoint
	"add a line segment, starting from current path endpoint to aPoint"
	
	self subclassResponsibility! !

!AthensPathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/7/2012 14:27'!
moveTo: aPoint
	" move command always starts a new contour "
	self subclassResponsibility! !

!AthensPathBuilder methodsFor: 'path commands' stamp: 'JochenRick 1/8/2014 06:27'!
reflectedCurveVia: cp2 to: aPoint
	"Add a reflected cubic bezier curve, starting from current path endpoint and ending at aPoint.
	The first control point is calculated as a reflection from the current point, if the last command was also a cubic bezier curve.
	Otherwise, the first control point is the current point. The second control point is cp2."
	
	self subclassResponsibility 	! !

!AthensPathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/7/2012 14:27'!
relative

	"next commands will use relative coordinates for all segment points"
	self subclassResponsibility! !

!AthensPathBuilder methodsFor: 'path commands' stamp: 'AliakseiSyrel 11/19/2015 14:59'!
string: aString font: aFont
	"Adds closed paths for the string to the current path"
	self subclassResponsibility! !

AthensPathBuilder subclass: #AthensSimplePathBuilder
	instanceVariableNames: 'absolute open contourStartPt xMin yMin xMax yMax pathStart lastSegment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensSimplePathBuilder commentStamp: '<historical>' prior: 0!
i building path containing from AthensPathSegment. 
the resulting path is backend neutral (while different backend may implement own builder which is more efficient)!

!AthensSimplePathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/21/2011 03:40'!
absolute
	absolute := true.! !

!AthensSimplePathBuilder methodsFor: 'private' stamp: 'IgorStasenko 3/26/2011 16:32'!
addSegment: aSegment
	lastSegment next: aSegment.
	lastSegment := aSegment.
 ! !

!AthensSimplePathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/7/2012 13:41'!
ccwArcTo: endPt angle: rot
	" angle should be specified in radians "
	^ self addSegment: (AthensCCWArcSegment new endPoint: (self toAbsolute: endPt) angle: rot).

! !

!AthensSimplePathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/26/2011 16:47'!
close
	"close the current contour"
	self addSegment: (AthensCloseSegment new point: contourStartPt).

	contourStartPt := nil.
	open := false! !

!AthensSimplePathBuilder methodsFor: 'creating path' stamp: 'IgorStasenko 4/18/2013 17:47'!
createPath:	aBlock

		"aBlock value: self ...."	

	aBlock value: self.
	
	^ pathStart! !

!AthensSimplePathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/26/2011 19:26'!
curveVia: cp1 and: cp2 to: aPoint
	"Cubic bezier curve"
	
	| pt1 pt2 pt3 |
	"Quad bezier curve"

	pt1 := self toAbsolute: cp1.
	pt2 := self toAbsolute: cp2.
	pt3 := self toAbsolute: aPoint.
	^ self addSegment: (
		AthensCubicSegment new 
			from: lastSegment endPoint 
			via: pt1
			and: pt2
			to: pt3)

! !

!AthensSimplePathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/26/2011 16:51'!
curveVia: cp1 to: aPoint
	| pt1 pt2 |
	"Quad bezier curve"

	pt1 := self toAbsolute: cp1.
	pt2 := self toAbsolute: aPoint.
	^ self addSegment: (
		AthensQuadSegment new 
			from: lastSegment endPoint 
			via: pt1 
			to: pt2)

! !

!AthensSimplePathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/7/2012 13:37'!
cwArcTo: endPt angle: rot
	" angle should be specified in radians "
	^ self addSegment: (AthensCWArcSegment new endPoint: (self toAbsolute: endPt) angle: rot).

! !

!AthensSimplePathBuilder methodsFor: 'initialize-release' stamp: 'IgorStasenko 4/18/2013 14:26'!
initialize
	"A new path always starts from implicit (moveTo:0@0) segment.
	If next segment is moveTo: , the point of already existing move segment will be changed,
	avoiding creating extra move segments.
	
	"
	absolute := false.
	contourStartPt := ZeroPoint.
	pathStart := lastSegment := (AthensMoveSegment new  point: ZeroPoint).! !

!AthensSimplePathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/26/2011 16:45'!
lineTo: aPoint

	^ self addSegment: (AthensLineSegment new point: (self toAbsolute: aPoint)).

! !

!AthensSimplePathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/26/2011 16:50'!
moveTo: aPoint
	" move command always starts a new contour "
	contourStartPt := self toAbsolute: aPoint.
	
	"collapse multiple moves to a single one"
	lastSegment isMove ifTrue: [
		lastSegment point: contourStartPt.
		^ self
	].

	self addSegment: (AthensMoveSegment new point: contourStartPt; reopen: open).
	open := true.! !

!AthensSimplePathBuilder methodsFor: 'accessing' stamp: 'IgorStasenko 3/26/2011 19:40'!
pathBounds
	^ xMin@yMin corner: xMax@yMax! !

!AthensSimplePathBuilder methodsFor: 'accessing' stamp: 'IgorStasenko 3/26/2011 19:50'!
pathStart
	^ pathStart! !

!AthensSimplePathBuilder methodsFor: 'path commands' stamp: 'JochenRick 1/8/2014 08:11'!
reflectedCurveVia: cp2 to: aPoint
	"Reflected cubic bezier curve"
	
	| pt1 pt2 pt3 |

	pt2 := self toAbsolute: cp2.
	pt1 := lastSegment isCubic
		ifTrue: [ lastSegment via2reflected ]
		ifFalse: [ lastSegment endPoint ].
	pt3 := self toAbsolute: aPoint.
	^ self addSegment: (
		AthensCubicSegment new 
			from: lastSegment endPoint 
			via: pt1
			and: pt2
			to: pt3)

! !

!AthensSimplePathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/21/2011 03:40'!
relative
	absolute := false.! !

!AthensSimplePathBuilder methodsFor: 'private' stamp: 'IgorStasenko 3/26/2011 16:43'!
toAbsolute: aPoint
	| pt |
	pt := absolute 
		ifTrue: [ aPoint ]
		ifFalse: [ lastSegment endPoint + aPoint ].

	"note the coordinate to calculate the path's bounding box"
	xMin ifNil: [
		xMin := xMax := pt x.
		yMin := yMax := pt y.	
		] 
	ifNotNil: [
		xMin := pt x min: xMin.
		yMin := pt y min: yMin.
		xMax := pt x max: xMax.
		yMax := pt y max: yMax.	
	].
	^ pt! !

Object subclass: #AthensPathConverter
	instanceVariableNames: 'sourcePath dest endPoint'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensPathConverter commentStamp: 'IgorStasenko 5/1/2011 23:14' prior: 0!
i am a base class for path conversion. 
i take a path as input and producing a path commands as output.

My default implementation just passing all path commands without changes to destination without any conversion

Example of use:

 
converter := AthensXYZConverter dest: (AthensPolygonPath new).
convertedPath := converter convert: sourcePath.

You can chain multiple converters:

basicConverter := AthensZYXConv dest: AthensPolygonPath new. 
compoundConverter := AthensABCConverter dest: basicConverter.

compoundConverter convert: somePath

in this case, an initial conversion is performed by instance of AthensABCConverter, and then conversion results are passed down to AthensZYXConv,
and then finally to instance of AthensPolygonPath.
!

AthensPathConverter subclass: #AthensBezierConverter
	instanceVariableNames: 'distanceTolerance angleTolerance'
	classVariableNames: 'CollinearityEps CurveAngleTolerance DistanceEps'
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensBezierConverter commentStamp: 'IgorStasenko 5/1/2011 21:50' prior: 0!
I converting all bezier segments to a sequence of line segments, by approximating the curve along its path.

This classs is used for generating strokes.!

!AthensBezierConverter class methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/29/2011 18:10'!
initialize

	CollinearityEps := 1e-30.
	DistanceEps := 1e-30.
	CurveAngleTolerance := 0.01.! !

AthensBezierConverter initialize!

!AthensBezierConverter methodsFor: 'helpers' stamp: 'IgorStasenko 5/1/2011 21:45'!
angleBetween: p1 and: p2 ifDegenerate: aBlock

" Calculate an angle (in radians) between two vectors. 
Evaluate a block, in case if calculation not possible because one of the vectors has zero length "

	| x1 y1 x2 y2 dot2 n2 |
	x1 := p1 x.
	y1 := p1 y.
	x2 := p2 x.
	y2 := p2 y.
	
	dot2 := x1 * x2 + (y1 * y2).
	dot2 := dot2 * dot2.
	
	n2 := (x1*x1 + (y1*y1)) * (x2*x2 + (y2*y2)).
	
	n2 = 0 ifTrue: [ ^ aBlock value ].
	
	^ (dot2 / n2) arcCos! !

!AthensBezierConverter methodsFor: 'converting path commands' stamp: 'IgorStasenko 5/1/2011 22:03'!
curveVia: pt1 to: pt2


	self	 recursiveBezier2_x1: endPoint x y1: endPoint y
		x2: pt1 x y2: pt1 y
		x3: pt2 x y3: pt2 y! !

!AthensBezierConverter methodsFor: 'initialize-release' stamp: 'IgorStasenko 4/29/2011 18:11'!
initialize

	distanceTolerance := 0.5.
	angleTolerance := 0.1 . ! !

!AthensBezierConverter methodsFor: 'testing' stamp: 'IgorStasenko 4/29/2011 18:09'!
isFlatBezier2_x1: x1 y1: y1 x2: x2 y2: y2 x3: x3 y3: y3

	| dx dy d da angle |
	
	dx := x3-x1.
	dy := y3-y1.
	
 	d := (((x2 - x3) * dy) - ((y2 - y3) * dx)) abs.

	d > CollinearityEps ifTrue: [
		
		"regular case"

		d*d <= (distanceTolerance * ( dx*dx + (dy*dy))) ifTrue: [
			
			angleTolerance < CurveAngleTolerance ifTrue: [ ^ true ].
			
			angle := self angleBetween: x2-x1 @ (y2-y1) and: x3-x2 @ (y3-y2)
				ifDegenerate: [ 0.0 ].
			
			"parallel. no need to proceed further"
			angle <= angleTolerance ifTrue: [ ^ true ]					
		]
	]
	ifFalse: [ 
		"collinear"
		da := dx*dx + (dy*dy).
		
		da = 0 
			ifTrue: [ d := (x1-x2) squared + (y1-y2) squared ]
			ifFalse: [
				d = ((x2 - x1)*dx + ((y2 - y1)*dy)) / da.

				(d > 0.0 and: [ d < 1.0 ] ) ifTrue: [ 
					"Simple collinear case, 1---2---3" 
					^ true
     				].
				d <= 0.0 
					ifTrue: [ d := (x1-x2) squared + (y1-y2) squared ]
					ifFalse: [
						d >= 1.0 
							ifTrue: [ d:= (x2-x3) squared + (y2-y3) squared ]
							ifFalse: [ d:= (x2 - x1 - (d*dx)) squared + (y2 - y1 - (d*dy)) squared ]
					].
			].

			d < self distanceToleranceSquared ifTrue: [ ^ true ]		
	].

	^ false! !

!AthensBezierConverter methodsFor: 'helpers' stamp: 'IgorStasenko 5/1/2011 21:47'!
recursiveBezier2_x1: x1 y1: y1 x2: x2 y2: y2 x3: x3 y3: y3

	"recursively subdive bezier curve as long as #isFlatBezier2.. answers false "

	(self isFlatBezier2_x1: x1 y1: y1 x2: x2 y2: y2 x3: x3 y3: y3) ifTrue: [
		
		dest 
			lineTo: x2 @ y2;
			lineTo: x3 @ y3
	] ifFalse: [
		| x12 y12 x23 y23 x123 y123 |	
	"calculate midpoints of line segments "
		x12 := (x1 + x2) * 0.5.
		y12 := (y1 + y2) * 0.5 .

		x23 := (x2 + x3) * 0.5 .
		y23 := (y2 + y3) * 0.5 .

		x123 := (x12 + x23) * 0.5.
		y123 := (y12 + y23) * 0.5.
		
		self recursiveBezier2_x1: x1 y1: y1 
			x2: x12 
			y2: y12 
			x3: x123 
			y3: y123.
			
		self recursiveBezier2_x1: x123 
			y1: y123
			x2: x23
			y2: y23 
			x3: x3 
			y3: y3.
	]
! !

!AthensPathConverter class methodsFor: 'instance creation' stamp: 'IgorStasenko 5/1/2011 22:18'!
dest: aDest

	^ self new dest: aDest! !

!AthensPathConverter class methodsFor: 'instance creation' stamp: 'IgorStasenko 5/1/2011 22:20'!
source: aSource dest: aDest
	" answer a converted path from source to dest, using my instance as a converter"
	
	^ self new source: aSource dest: aDest! !

!AthensPathConverter methodsFor: 'converting path commands' stamp: 'IgorStasenko 5/2/2011 05:18'!
close: aPoint 
	^ dest close: aPoint! !

!AthensPathConverter methodsFor: 'actions' stamp: 'IgorStasenko 5/1/2011 22:16'!
convert: aSourcePath
	
	" iterate over segments and do conversion "
	
	| segment |
	
	segment := aSourcePath.
	
	[ segment notNil ] whileTrue: [
		segment convertWith: self.
		endPoint := segment endPoint.
		segment := segment next.
	].

	^ dest finish! !

!AthensPathConverter methodsFor: 'converting path commands' stamp: 'IgorStasenko 4/29/2011 18:31'!
curveVia: pt1 to: pt2

	^ dest curveVia: pt1 to: pt2
! !

!AthensPathConverter methodsFor: 'accessing' stamp: 'IgorStasenko 5/1/2011 22:18'!
dest: anObject
	dest := anObject! !

!AthensPathConverter methodsFor: 'actions' stamp: 'IgorStasenko 5/1/2011 22:13'!
finish
	^ dest finish! !

!AthensPathConverter methodsFor: 'converting path commands' stamp: 'IgorStasenko 4/29/2011 18:36'!
lineTo: aPoint 
	^ dest lineTo: aPoint! !

!AthensPathConverter methodsFor: 'converting path commands' stamp: 'IgorStasenko 4/29/2011 18:28'!
moveTo: aPoint 
	^ dest moveTo: aPoint! !

!AthensPathConverter methodsFor: 'accessing' stamp: 'IgorStasenko 4/29/2011 18:18'!
source: aSource dest: aDest

	sourcePath := aSource.
	dest := aDest! !

Object subclass: #AthensPathSegment
	instanceVariableNames: 'next'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensPathSegment commentStamp: '<historical>' prior: 0!
The path segment is path building block.
Path is formed from list of connected path segments. At building stage, there is no container nor separately defined "path" object, just a linked list of segments.
Later the segments are accumulated in backend-specific path object, and the way how it is organized may vary.

My (sub)instances usually are not created directly by user, but instead by instance
of AthensPathBuilder.
Therefore, all my subclasses are considered private and implementation detail.!

AthensPathSegment subclass: #AthensArcSegment
	instanceVariableNames: 'endPoint angle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensArcSegment commentStamp: 'IgorStasenko 3/7/2012 13:41' prior: 0!
i am abstract class , representing an arc segment!

!AthensArcSegment methodsFor: 'accessors' stamp: 'IgorStasenko 3/7/2012 13:44'!
angle
	^ angle! !

!AthensArcSegment methodsFor: 'accessors' stamp: 'IgorStasenko 3/7/2012 13:31'!
endPoint
	^ endPoint! !

!AthensArcSegment methodsFor: 'accessors' stamp: 'IgorStasenko 3/7/2012 13:30'!
endPoint: pt angle: a

	endPoint := pt.
	angle := a! !

AthensArcSegment subclass: #AthensCCWArcSegment
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensCCWArcSegment commentStamp: '<historical>' prior: 0!
i represent a circular arc, connecting  
previous segment endpoint and my endpoing
of given angle, passing in counter-clockwise direction.!

!AthensCCWArcSegment methodsFor: 'visitor' stamp: 'IgorStasenko 4/18/2013 04:39'!
accept: aVisitor
	^ aVisitor ccwArcSegment: self! !

!AthensCCWArcSegment methodsFor: 'visitor' stamp: 'IgorStasenko 4/18/2013 05:21'!
sendCommandTo: aBuilder
	^ aBuilder ccwArcTo: endPoint angle: angle! !

AthensArcSegment subclass: #AthensCWArcSegment
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensCWArcSegment commentStamp: '<historical>' prior: 0!
i represent a circular arc, connecting  
previous segment endpoint and my endpoing
of given angle, passing in clockwise direction.!

!AthensCWArcSegment methodsFor: 'visitor' stamp: 'IgorStasenko 4/18/2013 04:39'!
accept: aVisitor
	^ aVisitor cwArcSegment: self! !

!AthensCWArcSegment methodsFor: 'visitor' stamp: 'IgorStasenko 4/18/2013 05:21'!
sendCommandTo: aBuilder
	^ aBuilder cwArcTo: endPoint angle: angle! !

AthensPathSegment subclass: #AthensCubicSegment
	instanceVariableNames: 'via1 via2 to'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensCubicSegment commentStamp: 'IgorStasenko 3/7/2012 13:32' prior: 0!
i represent a cubic Bezier path segment!

!AthensCubicSegment methodsFor: 'visitor' stamp: 'IgorStasenko 4/18/2013 04:39'!
accept: aVisitor
	^ aVisitor cubicBezierSegment: self! !

!AthensCubicSegment methodsFor: 'accessing' stamp: 'FernandoOlivero 2/14/2012 11:39'!
endPoint
	^ to! !

!AthensCubicSegment methodsFor: 'accessing' stamp: 'FernandoOlivero 2/14/2012 08:17'!
from: origin
	via: controlPoint
	and: secondControlPoint
	to: destination
	
	
	via1:= controlPoint.
	via2 := secondControlPoint.
	to := destination.! !

!AthensCubicSegment methodsFor: 'testing' stamp: 'JochenRick 1/8/2014 06:32'!
isCubic
	^true! !

!AthensCubicSegment methodsFor: 'visitor' stamp: 'IgorStasenko 4/18/2013 05:22'!
sendCommandTo: aBuilder
	^ aBuilder curveVia: via1 and: via2  to: to 
	! !

!AthensCubicSegment methodsFor: 'accessing' stamp: 'FernandoOlivero 2/14/2012 08:05'!
to

	^ to! !

!AthensCubicSegment methodsFor: 'accessing' stamp: 'FernandoOlivero 2/14/2012 08:09'!
via1
	^ via1! !

!AthensCubicSegment methodsFor: 'accessing' stamp: 'FernandoOlivero 2/14/2012 08:09'!
via2
	^ via2! !

!AthensCubicSegment methodsFor: 'accessing' stamp: 'JochenRick 1/8/2014 08:11'!
via2reflected
	^self endPoint * 2 - via2! !

AthensPathSegment subclass: #AthensEllipticalArcSegment
	instanceVariableNames: 'radii phi large sweep endPoint'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensEllipticalArcSegment commentStamp: '<historical>' prior: 0!
I representing an elliptical arc, using endpoint parametrization:

- initial endpoint (x0,y0), which comes from previous segment
- final endpoint (x1,y1)

- rh, rv , the radii of unrotated ellipse

- Large arc flag: true if more than 180 degrees of the arc is to be traversed (as measured on the unscaled circle), false otherwise

- Sweep flag: true if the arc is to be traversed in the counter-clockwise direction, false otherwise

(not functional yet)!

!AthensEllipticalArcSegment methodsFor: 'visitor' stamp: 'IgorStasenko 4/18/2013 04:42'!
accept: aVisitor
	^ aVisitor ellipticalArcSegment: self! !

AthensPathSegment subclass: #AthensLineSegment
	instanceVariableNames: 'point'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensLineSegment commentStamp: '<historical>' prior: 0!
i represent a line segment,
created using 
	#lineTo: 
command of AthensPathBuilder!

AthensLineSegment subclass: #AthensCloseSegment
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensCloseSegment commentStamp: '<historical>' prior: 0!
i represent close segment (when user issuing #close command to path builder)!

!AthensCloseSegment methodsFor: 'visitor' stamp: 'IgorStasenko 4/18/2013 04:40'!
accept: aVisitor
	^ aVisitor closeSegment: self! !

!AthensCloseSegment methodsFor: 'visitor' stamp: 'IgorStasenko 4/18/2013 05:22'!
sendCommandTo: aBuilder
	^ aBuilder close
	! !

!AthensLineSegment methodsFor: 'visitor' stamp: 'IgorStasenko 4/18/2013 04:41'!
accept: aVisitor
	^ aVisitor lineSegment: self! !

!AthensLineSegment methodsFor: 'accessing' stamp: 'IgorStasenko 3/26/2011 16:21'!
endPoint
	^ point! !

!AthensLineSegment methodsFor: 'accessing' stamp: 'IgorStasenko 3/26/2011 16:16'!
point

	^ point! !

!AthensLineSegment methodsFor: 'accessing' stamp: 'IgorStasenko 3/26/2011 16:16'!
point: anObject

	point := anObject! !

!AthensLineSegment methodsFor: 'visitor' stamp: 'IgorStasenko 4/18/2013 05:23'!
sendCommandTo: aBuilder
	^ aBuilder lineTo: point
	! !

!AthensLineSegment methodsFor: 'visitor' stamp: 'NicolaiHess 6/24/2014 20:58'!
visitWith: visitor

	visitor visitLineSegment:  self! !

AthensLineSegment subclass: #AthensMoveSegment
	instanceVariableNames: 'closed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensMoveSegment commentStamp: 'IgorStasenko 5/2/2011 05:35' prior: 0!
Move segment always indicates starting of a new contour.
If contour ends with close command, then my insntance will have 'closed' flag set to true.

See AthensPathBuilder , which is responsible for building paths.!

!AthensMoveSegment methodsFor: 'visitor' stamp: 'IgorStasenko 4/18/2013 04:40'!
accept: aVisitor
	^ aVisitor moveSegment: self! !

!AthensMoveSegment methodsFor: 'accessing' stamp: 'IgorStasenko 5/2/2011 05:37'!
closed: aBoolean
	closed := aBoolean! !

!AthensMoveSegment methodsFor: 'testing' stamp: 'IgorStasenko 5/2/2011 05:37'!
isClosed
	^ closed == true! !

!AthensMoveSegment methodsFor: 'testing' stamp: 'IgorStasenko 3/27/2011 17:48'!
isMove
	^ true! !

!AthensMoveSegment methodsFor: 'accessing' stamp: 'FernandoOlivero 1/15/2012 14:59'!
reopen: aBoolean
	closed := aBoolean! !

!AthensMoveSegment methodsFor: 'visitor' stamp: 'IgorStasenko 4/18/2013 05:23'!
sendCommandTo: aBuilder
	^ aBuilder moveTo: point
	! !

!AthensMoveSegment methodsFor: 'visitor' stamp: 'NicolaiHess 6/24/2014 20:57'!
visitWith: visitor

	visitor visitMoveSegment: self! !

!AthensPathSegment methodsFor: 'visitor' stamp: 'IgorStasenko 4/18/2013 04:38'!
accept: aVisitor
	self subclassResponsibility ! !

!AthensPathSegment methodsFor: 'visitor' stamp: 'IgorStasenko 4/29/2011 18:13'!
convertWith: anObject
	self subclassResponsibility! !

!AthensPathSegment methodsFor: 'iterating' stamp: 'IgorStasenko 4/18/2013 04:52'!
do: aBlock
	| segment |
	
	segment := self.
	
	[  aBlock value: segment. segment := segment next. segment notNil ] whileTrue.! !

!AthensPathSegment methodsFor: 'testing' stamp: 'JochenRick 1/8/2014 06:31'!
isCubic
	^ false! !

!AthensPathSegment methodsFor: 'testing' stamp: 'IgorStasenko 3/26/2011 16:44'!
isMove
	^ false! !

!AthensPathSegment methodsFor: 'accessing' stamp: 'IgorStasenko 3/26/2011 16:14'!
next

	^ next! !

!AthensPathSegment methodsFor: 'accessing' stamp: 'IgorStasenko 3/26/2011 16:14'!
next: anObject

	next := anObject! !

!AthensPathSegment methodsFor: 'iterating' stamp: 'IgorStasenko 4/18/2013 05:20'!
sendCommandsTo: aBuilder

	self do: [ :segment |
		segment sendCommandTo: aBuilder
	]! !

!AthensPathSegment methodsFor: 'accessing' stamp: 'IgorStasenko 4/18/2013 21:51'!
size
	"answer 1 + the number of segments next to receiver"
	
	| segment count |
	segment := self.
	count := 0.
	[ segment notNil ] whileTrue: [  count := count + 1. segment := segment next ].

	^ count.! !

!AthensPathSegment methodsFor: 'visitor' stamp: 'IgorStasenko 4/13/2013 17:33'!
visitWith: visitor

	self subclassResponsibility! !

AthensPathSegment subclass: #AthensQuadSegment
	instanceVariableNames: 'via to'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensQuadSegment commentStamp: 'IgorStasenko 3/7/2012 13:32' prior: 0!
i represent a quadric Bezier path segment.

Any quadratic spline can be expressed as a cubic (where the cubic term is zero). The end points of the cubic will be the same as the quadratic's.

    CP0 = QP0
    CP3 = QP2 

The two control points for the cubic are:

    CP1 = QP0 + 2/3 *(QP1-QP0)
    CP2 = QP2 + 2/3 *(QP1-QP2) !

!AthensQuadSegment methodsFor: 'visitor' stamp: 'IgorStasenko 4/18/2013 04:40'!
accept: aVisitor
	^ aVisitor quadricBezierSegment: self! !

!AthensQuadSegment methodsFor: 'accessing' stamp: 'IgorStasenko 3/26/2011 16:52'!
endPoint
	^ to! !

!AthensQuadSegment methodsFor: 'initialize-release' stamp: 'IgorStasenko 3/26/2011 16:52'!
from: pt0 via: pt1 to: p

	via := pt1.
	to := p.! !

!AthensQuadSegment methodsFor: 'visitor' stamp: 'IgorStasenko 4/18/2013 05:22'!
sendCommandTo: aBuilder
	^ aBuilder curveVia: via to: to 
	! !

!AthensQuadSegment methodsFor: 'accessing' stamp: 'IgorStasenko 3/26/2011 16:52'!
to

	^ to! !

!AthensQuadSegment methodsFor: 'accessing' stamp: 'IgorStasenko 3/26/2011 16:52'!
via

	^ via! !

!AthensQuadSegment methodsFor: 'visitor' stamp: 'NicolaiHess 6/24/2014 20:58'!
visitWith: visitor

	visitor visitQuadSegment: self! !

Object subclass: #AthensPathSegmentConverter
	instanceVariableNames: 'dest endPoint contourStartPt'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensPathSegmentConverter commentStamp: '<historical>' prior: 0!
this class is subject of changes/removal. do not use it.!

AthensPathSegmentConverter subclass: #AthensCurveFlattener
	instanceVariableNames: 'transform inverseTransform lengthTolerance lengthToleranceSquared'
	classVariableNames: 'CollinearityEps CurveAngleToleranceEpsilon CurveCollinearityEpsilon CurveDistanceEpsilon SubdivisionLimit'
	poolDictionaries: ''
	category: 'Athens-Core-PathsGeometry'!

!AthensCurveFlattener commentStamp: '<historical>' prior: 0!
This converter flattens all curves by converting them to series of lines.
This means that path can be rendered using simple polygon rendering technique
and nothing else.!

!AthensCurveFlattener class methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/26/2013 17:40'!
initialize
	CurveCollinearityEpsilon := 1e-30 asFloat.
	CurveDistanceEpsilon := 1e-30 asFloat.
	CurveAngleToleranceEpsilon := 0.01.
	SubdivisionLimit := 10.! !

AthensCurveFlattener initialize!

!AthensCurveFlattener methodsFor: 'subdivision callbacks' stamp: 'IgorStasenko 4/29/2013 21:48'!
accountForAngleTolerance
	" It's important only when we want to draw an equidistant curve, that is, a stroke of considerable width. If we don't need to draw a stroke or the stroke width is one pixel or less, the distance criterion works quite well. "
	^ false.! !

!AthensCurveFlattener methodsFor: 'subdivision callbacks' stamp: 'IgorStasenko 4/29/2013 21:48'!
angleTolerance
	^ CurveAngleToleranceEpsilon ! !

!AthensCurveFlattener methodsFor: 'subdivision callbacks' stamp: 'IgorStasenko 4/25/2013 00:59'!
curveCollinearityEpsilon

	^ CurveCollinearityEpsilon ! !

!AthensCurveFlattener methodsFor: 'path commands' stamp: 'IgorStasenko 4/26/2013 17:34'!
curveVia: via1 and: via2 to: end 
	| pt1 pt2 pt3 pt4 curve |
	
	pt1 := transform transform: endPoint.
	pt2 := transform transform: via1.
	pt3 := transform transform: via2.
	pt4 := transform transform: end.
	
	endPoint := end.

	curve := AthensCubicBezier new
		x1: pt1 x;
		y1: pt1 y;
		x2: pt2 x;
		y2: pt2 y;
		x3: pt3 x;
		y3: pt3 y;
		x4: pt4 x;
		y4: pt4 y.
		
	curve recursiveSubDiv: self level: 0.
	self lineTo: endPoint.
	
	! !

!AthensCurveFlattener methodsFor: 'path commands' stamp: 'IgorStasenko 4/18/2013 21:47'!
curveVia: via to: end 
	| pt1 pt2 pt3 |
	
	pt1 := transform transform: endPoint.
	pt2 := transform transform: via.
	pt3 := transform transform: end.
	
	endPoint := end.

	self flattenQuadBezier: pt1 x y1: pt1 y x2: pt2 x y2: pt2 y x3: pt3 x y3: pt3 y.
! !

!AthensCurveFlattener methodsFor: 'accessing' stamp: 'IgorStasenko 4/18/2013 21:59'!
defaultTolerance
	^ 1! !

!AthensCurveFlattener methodsFor: 'subdivision callbacks' stamp: 'IgorStasenko 5/2/2013 17:43'!
distanceToleranceSquared
	^ self lengthToleranceSquared ! !

!AthensCurveFlattener methodsFor: 'helpers' stamp: 'IgorStasenko 4/19/2013 22:17'!
flattenCubicBezier: x1 y1: y1 x2: x2 y2: y2 x3: x3 y3: y3 x4: x4 y4: y4
! !

!AthensCurveFlattener methodsFor: 'public API' stamp: 'ClementBera 6/28/2013 10:29'!
flattenPath: aPath transform: aTransformation

	"This is an entry point for flattening path.
	An additional argument, transform is an Affine matrix, used to map path geometry to screen,
	and therefore used to calculate the error tolerance for path subdivisions"
	
	^ self flattenPath: aPath transform: aTransformation toleranceMultiplier: 1 ! !

!AthensCurveFlattener methodsFor: 'public API' stamp: 'IgorStasenko 4/18/2013 17:48'!
flattenPath: aPath transform: aTransform toleranceMultiplier: aToleranceMultiplier

	"This is an entry point for flattening path (converting all curves into line segments by approximating them).
	
	- transform is an Affine matrix, used to map path geometry to screen,
	and therefore used to calculate the error tolerance for path subdivisions
	
	
	- a tolerance multiplier is a number .. which
		if = 1 , the default error tolerance is used,
		if > 1 you will get more coarse approximation
		if < 1 you will get more finer approximation
	
	"
	
	transform := aTransform.
	lengthTolerance := aToleranceMultiplier.
	lengthToleranceSquared := lengthTolerance squared.
	
! !

!AthensCurveFlattener methodsFor: 'helpers' stamp: 'IgorStasenko 4/18/2013 23:34'!
flattenQuadBezier: x1 y1: y1 x2: x2 y2: y2 x3: x3 y3: y3

	"Recursively subdive quadric bezier curve as long as #isFlatBezier.. answers false "

	"The points here is unboxed intentionally to avoid generating extra garbage
	(which contributes to performance loss)"

	(self isFlatQuadBezier: x1 y1: y1 x2: x2 y2: y2 x3: x3 y3: y3) ifTrue: [
		| midx midy |
		midx := (x2 + x1 + x2+x3) * 0.25 .  "mid ( mid(pt1,pt2), mid(pt2,pt3)) "
		midy := (y2 + y1 + y2+y3) * 0.25 . 

		dest 
			lineTo: (self inverseTransform: midx @ midy);
			lineTo: (self inverseTransform: x3 @ y3)
			
	"	dest 
			lineTo: (self inverseTransform: x2 @ y2);
			lineTo: (self inverseTransform: x3 @ y3)
	"
	
	] ifFalse: [
		| x12 y12 x23 y23 x123 y123 |	
	"calculate midpoints of line segments "
		x12 := (x1 + x2) * 0.5.
		y12 := (y1 + y2) * 0.5 .

		x23 := (x2 + x3) * 0.5 .
		y23 := (y2 + y3) * 0.5 .

		x123 := (x12 + x23) * 0.5.
		y123 := (y12 + y23) * 0.5.
		
		self flattenQuadBezier: x1 y1: y1 
			x2: x12 
			y2: y12 
			x3: x123 
			y3: y123.
			
		self flattenQuadBezier: x123 
			y1: y123
			x2: x23
			y2: y23 
			x3: x3 
			y3: y3.
	]
! !

!AthensCurveFlattener methodsFor: 'initialize-release' stamp: 'IgorStasenko 4/18/2013 17:58'!
initialize

	super initialize.
	
	transform := AthensAffineTransform new. "identity"
	self toleranceMultiplier: self defaultTolerance! !

!AthensCurveFlattener methodsFor: 'helpers' stamp: 'IgorStasenko 4/18/2013 18:06'!
inverseTransform: aPoint 
	^ transform inverseTransform: aPoint! !

!AthensCurveFlattener methodsFor: 'helpers' stamp: 'IgorStasenko 4/18/2013 22:33'!
isFlatQuadBezier: x1 y1: y1 x2: x2 y2: y2 x3: x3 y3: y3

	| dx dy d da |
	
	dx := x3-x1.
	dy := y3-y1.
	
	"This is the area of triangle enclosing curve * 2"
 	d := (((x2 - x3) * dy) - ((y2 - y3) * dx)) abs.

	d > (self lengthToleranceSquared ) ifTrue: [
		
		"Non-collinear case (regular one)"
		| dot |

		"if dot product is close to zero, that means we having flat curve"			
		dot := ( (x2-x1)*(x2-x3) + ((y1-y2)*(y2-y3)) ) abs.
		dot < (self lengthToleranceSquared * 0.5 ) ifTrue: [  ^ true ].
	]
	ifFalse: [ 
		"collinear"
		da := dx*dx + (dy*dy).
		
		da = 0 "end points coincide"
			ifTrue: [ d := (x1-x2) squared + (y1-y2) squared  "pointy case" ]
			ifFalse: [
				"the control point lies on line between endpoints?"
				d := ((x2 - x1)*dx + ((y2 - y1)*dy)) / da.

				(d > 0.0 and: [ d < 1.0 ] ) ifTrue: [ 
					"Simple collinear case, 1---2---3" 
					^ true
     				].
				d <= 0.0 
					ifTrue: [ d := (x1-x2) squared + (y1-y2) squared ]
					ifFalse: [
						d >= 1.0 
							ifTrue: [ d:= (x2-x3) squared + (y2-y3) squared ]
							ifFalse: [ d:= (x2 - x1 - (d*dx)) squared + (y2 - y1 - (d*dy)) squared ]
					].
			].

			d < (self lengthToleranceSquared) ifTrue: [ ^ true ]		
	].

	^ false! !

!AthensCurveFlattener methodsFor: 'accessing' stamp: 'IgorStasenko 4/18/2013 17:49'!
lengthTolerance

	^ lengthTolerance! !

!AthensCurveFlattener methodsFor: 'accessing' stamp: 'IgorStasenko 4/18/2013 17:49'!
lengthToleranceSquared

	^ lengthToleranceSquared! !

!AthensCurveFlattener methodsFor: 'path commands' stamp: 'IgorStasenko 4/18/2013 21:46'!
lineTo: aPoint 

	endPoint := aPoint.
	
	^ dest lineTo: aPoint! !

!AthensCurveFlattener methodsFor: 'subdivision callbacks' stamp: 'IgorStasenko 4/25/2013 01:14'!
lineToX: trX y: trY

	^ dest lineTo: (self inverseTransform: trX @ trY) ! !

!AthensCurveFlattener methodsFor: 'path commands' stamp: 'IgorStasenko 4/18/2013 18:04'!
moveTo: aPoint 

	contourStartPt := endPoint := aPoint.
	
	^ dest moveTo: aPoint! !

!AthensCurveFlattener methodsFor: 'subdivision callbacks' stamp: 'IgorStasenko 4/29/2013 21:32'!
overCuspLimit: angleInRadians
	 "if(m_cusp_limit !!= 0.0)
                {
                    if(da1 > m_cusp_limit)
                    {
	
	^ angleInRadians > 1.01"
	^ false. ! !

!AthensCurveFlattener methodsFor: 'visiting' stamp: 'IgorStasenko 4/18/2013 17:24'!
quadricBezierSegment: segment
	| pt1 pt2 pt3 |
	
	pt1 := transform transform: endPoint.
	pt2 := transform transform: segment via.
	pt3 := transform transform: segment endPoint.
	
	endPoint := segment endPoint.

	self flattenQuadBezier: pt1 x y1: pt1 y x2: pt2 x y2: pt2 y x3: pt3 x y3: pt3 y.
! !

!AthensCurveFlattener methodsFor: 'subdivision callbacks' stamp: 'IgorStasenko 5/2/2013 17:43'!
subdivisionLimit
	"max number of recursive subdivisions for single curve"
	^ SubdivisionLimit ! !

!AthensCurveFlattener methodsFor: 'accessing' stamp: 'IgorStasenko 4/18/2013 17:56'!
toleranceMultiplier: aToleranceMultiplier

	"	
	A tolerance multiplier is a number .. which
		if = 1 , the default error tolerance is used,
		if > 1 you will get more coarse approximation
		if < 1 you will get more finer approximation
	
	"
	
	lengthTolerance := aToleranceMultiplier.
	lengthToleranceSquared := lengthTolerance squared.
	
! !

!AthensCurveFlattener methodsFor: 'accessing' stamp: 'IgorStasenko 4/18/2013 17:56'!
transform: aTransform 
	"
	- transform is an Affine matrix, used to map path geometry to screen,
	and therefore used to calculate the error tolerance for path subdivisions
	
	
	- a tolerance multiplier is a number .. which
		if = 1 , the default error tolerance is used,
		if > 1 you will get more coarse approximation
		if < 1 you will get more finer approximation
	
	"
	
	transform := aTransform.
	
! !

!AthensPathSegmentConverter methodsFor: 'converting' stamp: 'IgorStasenko 4/18/2013 17:50'!
convert: aPath
	"pipe the path segments to destination, converting
	segments on the way"
	
	contourStartPt := endPoint := 0@0.
	
	aPath sendCommandsTo: self.
	
	
	! !

!AthensPathSegmentConverter methodsFor: 'accessing' stamp: 'IgorStasenko 4/18/2013 17:39'!
dest: aDest
	dest := aDest! !

Object subclass: #AthensPolygon
	instanceVariableNames: 'contours currentContour'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensPolygon commentStamp: 'IgorStasenko 12/20/2011 18:38' prior: 0!
i am special kind of shape which using only straight line segments, unlike from more generic paths.!

!AthensPolygon methodsFor: 'conversion' stamp: 'IgorStasenko 5/1/2011 22:11'!
finish
	"Finally convert contours to arrays"
	
	contours := contours collect: [:ea | ea asArray ]! !

!AthensPolygon methodsFor: 'conversion' stamp: 'IgorStasenko 4/29/2011 18:32'!
lineTo: aPoint 
	currentContour add: aPoint! !

!AthensPolygon methodsFor: 'conversion' stamp: 'IgorStasenko 4/29/2011 18:29'!
moveTo: aPoint 
	" create a new contour "
	
	self newContour.
	currentContour add: aPoint.! !

!AthensPolygon methodsFor: 'conversion' stamp: 'IgorStasenko 4/29/2011 18:30'!
newContour
	contours ifNil: [ contours := OrderedCollection new ].
	
	currentContour := contours add: OrderedCollection new.! !

!AthensPolygon methodsFor: 'drawing' stamp: 'IgorStasenko 5/2/2011 04:48'!
paintFillsUsing: aPaint on: anAthensCanvas 
	
	"This method is a part of rendering dispatch  Canvas->receiver->paint"
	
	^ aPaint fillPolygon: self on: anAthensCanvas! !

Object subclass: #AthensPolygonTester
	instanceVariableNames: 'polygon'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensPolygonTester commentStamp: '<historical>' prior: 0!
I implementing a simple and efficient algorithm to test whether given point
lies withing a polygon or not.

A polygon is a collection of points 1...size.!

!AthensPolygonTester methodsFor: 'testing' stamp: 'IgorStasenko 4/19/2013 16:29'!
includesPoint: aPoint
"
Thanks to Google and Randolph Franklin i don't have to reinvent this very simple algorithm.
See [ 1 ] for details, copyrights etc.

[1] http://www.ecse.rpi.edu/Homepages/wrf/Research/Short_Notes/pnpoly.html

"
	| inside testX testY i j size |
	
	inside := false.
	
	i := 1.
	size := polygon size.
	j := size.
	testX := aPoint x.
	testY := aPoint y.
	
	[ i <= size ] whileTrue: [  | pi pj |
		pi := polygon at: i.
		pj := polygon at: j. 
	
		(((pi y > testY) ~= (pj y > testY)) and: [ 
			testX < 
			(
				pj x - pi x
				* (testY - pi y) 
				/ ( pj y - pi y )
				+ pi x 
			)
			 ]) ifTrue: [ inside := inside not ].
	
		j := i.
		i := i + 1.
	 ].
	^ inside
	! !

!AthensPolygonTester methodsFor: 'accessing' stamp: 'IgorStasenko 4/19/2013 15:56'!
polygon: poly

	polygon := poly! !

Object subclass: #AthensSurface
	instanceVariableNames: 'currentCanvas'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core'!

!AthensSurface commentStamp: '<historical>' prior: 0!
I representing a surface, where all drawing operations will happen.

I also having a number of factory methods to create paints, paths and other objects involved in drawing.

AthensSurface is an abstract, while subclasses implement a specific kind of surface for one or another backend.

The primary role of AthensSurface class is to define a public protocol for all Athens surfaces, which can be used by applications which using Athens framework.

To get a new surface, use:

<one of my subclasses> extent: x@y

for surfaces which don't need to have dimensions specified,
it would just #new.!

!AthensSurface class methodsFor: 'instance creation' stamp: 'sig 2/29/2012 21:43'!
extent: aPoint
	"Create a new surface with given extent. 
	Note that some surfaces may not support this method, since they may represent an infinite surface."
	
	self subclassResponsibility! !

!AthensSurface methodsFor: 'converting' stamp: 'sig 2/29/2012 21:51'!
asForm
	"Answer a Form , which contains a bits, converted from surface "
	self subclassResponsibility! !

!AthensSurface methodsFor: 'caching' stamp: 'IgorStasenko 10/12/2012 03:22'!
cacheAt: anObject ifAbsentPut: aBlock
	"Answer an object from surface's cache identified by anObject,
	if there is no cached object under such identifier, evaluate a block
	and put it into cache. Then answer the result of evaluation.
	A surface using identity comparison for object identifiers.
	"

	self subclassResponsibility! !

!AthensSurface methodsFor: 'public' stamp: 'IgorStasenko 12/20/2011 15:38'!
clear
	"clear the surface"
	
	self subclassResponsibility! !

!AthensSurface methodsFor: 'public' stamp: 'IgorStasenko 9/1/2012 20:06'!
clear: clearColor
	"clear the surface"
	
	self subclassResponsibility! !

!AthensSurface methodsFor: 'clipping' stamp: 'IgorStasenko 4/21/2011 14:47'!
clipBy: aRectangle during: aBlockClosure 
	self subclassResponsibility.! !

!AthensSurface methodsFor: 'paints' stamp: 'IgorStasenko 3/8/2012 13:46'!
createFormPaint: aForm 
	self subclassResponsibility.! !

!AthensSurface methodsFor: 'paints' stamp: 'IgorStasenko 4/12/2013 09:59'!
createLinearGradient: colorRamp start: pt1 stop: pt2
 
	"This protocol is deprecated. Use #createLinearGradient: start:stop: instead"
	
	
	self shouldNotImplement ! !

!AthensSurface methodsFor: 'paths' stamp: 'IgorStasenko 3/26/2011 19:32'!
createPath: aPathBuilder
	"Create a path from provided path builder instance"
	self subclassResponsibility! !

!AthensSurface methodsFor: 'paints' stamp: 'IgorStasenko 4/27/2012 12:17'!
createRadialGradient: colorRamp center: aCenter radius: aRadius
	"by default, focal point coincede with center"
	^ self createRadialGradient: colorRamp center: aCenter radius: aRadius focalPoint: aCenter
! !

!AthensSurface methodsFor: 'paints' stamp: 'IgorStasenko 4/27/2012 12:16'!
createRadialGradient: colorRamp center: aCenter radius: aRadius focalPoint: fp
 
	self subclassResponsibility! !

!AthensSurface methodsFor: 'paths' stamp: 'FernandoOlivero 1/13/2012 03:08'!
createRectanglePath: aRectangle 
	^ aRectangle! !

!AthensSurface methodsFor: 'paints' stamp: 'AliakseiSyrel 11/27/2015 00:20'!
createShadowPaint: aColor 
	 "Answer an instance of AthensPaint, valid for use with given surface"
	
	self subclassResponsibility! !

!AthensSurface methodsFor: 'paints' stamp: 'IgorStasenko 12/20/2011 15:25'!
createSolidColorPaint: aColor 
	 "Answer an instance of AthensPaint, valid for use with given surface"
	
	self subclassResponsibility! !

!AthensSurface methodsFor: 'paints' stamp: 'IgorStasenko 4/27/2012 11:20'!
createStrokePaintFor: aPaint
	 "Answer an instance of AthensPaint, valid for use as stroke paint on receiver,
	using an argument, paint for fills"
	
	self subclassResponsibility! !

!AthensSurface methodsFor: 'masking' stamp: 'IgorStasenko 3/27/2011 18:15'!
disableMask
	self subclassResponsibility! !

!AthensSurface methodsFor: 'public' stamp: 'IgorStasenko 3/7/2012 15:06'!
drawDuring: aBlock

	"You may draw on receiver only when inside a block and only using provided canvas object.
	This ensures releasing system resources used after finishing drawing"


	currentCanvas ifNotNil: [ self attemptToRecurseDrawing ].
	
	[
		currentCanvas := self newCanvas.
		aBlock value: currentCanvas.
	] ensure: [
		self releaseCanvas.
		currentCanvas := nil.
	].! !

!AthensSurface methodsFor: 'masking' stamp: 'IgorStasenko 3/27/2011 18:15'!
enableMask
	self subclassResponsibility! !

!AthensSurface methodsFor: 'rendering dispatch' stamp: 'NicolaiHess 6/24/2014 21:08'!
fillPath: aPath withSolidColor: aColor 

	self subclassResponsibility! !

!AthensSurface methodsFor: 'rendering dispatch' stamp: 'IgorStasenko 8/30/2013 16:38'!
fillRectangle: aRectangle withSolidColor: aColor

	self subclassResponsibility! !

!AthensSurface methodsFor: 'caching' stamp: 'IgorStasenko 10/12/2012 03:56'!
flushCacheAt: anObject

	"Flush (delete) any cached value(s) identified by given object, anObject.
	A surface using identity comparison for object identifiers.

	Answer receiver.
	"

	self subclassResponsibility! !

!AthensSurface methodsFor: 'masking' stamp: 'IgorStasenko 3/27/2011 18:16'!
maskEnabled
	self subclassResponsibility! !

!AthensSurface methodsFor: 'private' stamp: 'IgorStasenko 3/21/2011 02:35'!
newCanvas
	"Answer a preinitialized instance of AthensCanvas.
	Private to receiver and its subclasses, override seldom"
	^ AthensCanvas on: self
! !

!AthensSurface methodsFor: 'private' stamp: 'IgorStasenko 3/22/2011 22:40'!
releaseCanvas
	currentCanvas := nil! !

Object subclass: #AthensTransform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core'!

!AthensTransform commentStamp: 'IgorStasenko 3/7/2012 14:41' prior: 0!
My instances represent an interface for coordinate system transformation defined by 3x3 matrix of following kind:

| sx   shx   x   |
| shy sy     y   |
|  w0  w1  w2 |

sx and sy define scaling in the x and y directions, respectively;
shx and shy define shearing in the x and y directions, respectively;
tx and ty define translation in the x and y directions, respectively.

AthensSurface are responsible for providing specific implementation 
of me, which is most appropriate for backend its using.

My internal representation isnt available for manipulation, for example dont assume i'm a matrix. In short, do not copy my state or modify it directly, instead use methods such as #loadAffineTransformation:, and talk to my surface.

My subclasses should implement a common protocol for applying generic types of affine transformations
on coordinate system:
- translate
- rotate
- shear
- scale
- matrix multiply
- matrix load

Surface must support 4 kinds of transformations of coordinate system(s):
- path-to-surface (#pathTransform) to transform path coordinates to surface space
- image-to-surface (#imageTransform) to transform between user coordinates and surface pixels

- fill-paint-to-user (#fillTransform)
- stroke-paint-to-user (#strokeTransform)

Given a (fill or stroke) paint-to-user transformation Tp and user-to-surface transformation Tu, the paint color and alpha of a pixel to be drawn with surface coordinates (x, y) is defined by mapping its center point (x + 1/2, y + 1/2) through the inverse transformation (Tu * Tp)^-1 , resulting in a sample point in the paint coordinate space.

All transformations, except image-to-surface , ignoring w0 , w1 and w2 values and always assume them set to { 0 , 0 , 1 } respectively.

For accessing a particular kind of transformation, send message to canvas, i.e.:

canvas pathTransform translateBy: 10@10.

It is safe to store transformation in temporary variable, as long as canvas is valid.
!

AthensTransform subclass: #AthensAffineTransform
	instanceVariableNames: 'sx shx x shy sy y'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Matrices'!

!AthensAffineTransform commentStamp: '<historical>' prior: 0!
I support the same protocol than AthensTransform,
but in addition I store a plain 2x3 matrix with state accessible at any moment.

This is different to AthensTransform because it does not expose its internal storage, because it can be backend specific.!

!AthensAffineTransform methodsFor: 'transformations' stamp: 'IgorStasenko 10/21/2011 12:55'!
clearTranslation
	"reset a translation from matrix, leaving only scale and rotation"
	
	x := y := 0.0! !

!AthensAffineTransform methodsFor: 'initialize-release' stamp: 'IgorStasenko 3/26/2011 02:09'!
initialize
	self loadIdentity! !

!AthensAffineTransform methodsFor: 'vector-transform' stamp: 'NicolaiHess 11/25/2014 23:25'!
inverseTransform: aPoint 
"  (we assume matrix is never degenerate)"
	| px py y0 x0 |
	
	px := aPoint x.
	py := aPoint y.
	"for rotations around +/- half pi"
	(sx = 0 and:[ sy = 0]) ifTrue:[
		^ self inverseTransformHalfPi: aPoint.
		].
	"for rotations around +/- pi or 0"
	(shx = 0 and:[ shy = 0]) ifTrue:[
		^ self inverseTransformPiOrZero: aPoint.
		].
    "otherwise:
     Given straight transformation formulae:
		
	px := x0*sx + y0*shx + x.
	py := x0*shy + y0*sy + y.
	
	we doing inverse one, trying to find x0,y0 for rest of given variables (px,py,x,y,sx,sy,shx,shy).
	
	x0 := px - x - (shx*y0) / sx.
	y0 := py - y - (shy*x0) / sy.
	
	y0 := py - y - (shy*(px - x - (shx*y0) / sx)) / sy.

	sy * y0  == py - y - (shy*(px - x - (shx*y0) / sx)).

	sx * sy * y0  == (sx*(py - y)) - (shy*(px - x - (shx*y0))).

	sx * sy * y0  == sx*py - (sx*y) - (shy*px) + (shy*x) + (shy*shx*y0).

	(sx * sy * y0) - (shy*shx*y0)  == sx*py - (sx*y) - (shy*px) + (shy*x) .
	
	y0* ((sx * sy) - (shy*shx))  == sx*py - (sx*y) - (shy*px) + (shy*x) .

"
	y0  := sx*py - (sx*y) - (shy*px) + (shy*x) / ((sx * sy) - (shy*shx)).
	x0 := px - x - (shx*y0) / sx.

	^ x0@y0
	! !

!AthensAffineTransform methodsFor: 'private' stamp: 'NicolaiHess 11/25/2014 23:20'!
inverseTransformHalfPi: aPoint
	"assume sx,sy = 0
		
	px := y0*shx + x.
	py := x0*shy + y.
	
	we doing inverse one, trying to find x0,y0 for rest of given variables (px,py,x,y,sx,sy,shx,shy)"
	| px py y0 x0 |
	
	px := aPoint x.
	py := aPoint y.

	x0 := py - y  / shy.
	y0 := px - x  / shx.

	^ x0@y0
	! !

!AthensAffineTransform methodsFor: 'private' stamp: 'NicolaiHess 11/25/2014 23:22'!
inverseTransformPiOrZero: aPoint
	"assume shx,shy = 0
		
	px := x0*sx + x.
	py := y0*sy + y.
	we doing inverse one, trying to find x0,y0 for rest of given variables (px,py,x,y,sx,sy,shx,shy)"
	
	| px py y0 x0 |
	
	px := aPoint x.
	py := aPoint y.

	x0 := px - x / sx.
	y0 := py - y / sy.

	^ x0@y0
	! !

!AthensAffineTransform methodsFor: 'accessing' stamp: 'NicolaiHess 11/25/2014 23:32'!
inverted
	"answer an inverse transformation of receiver"
	| det |
	det := sx * sy - (shx * shy).
	^ self class new
		sx: sy / det;
		sy: sy / det;
		shx: shx * -1 / det;
		shy: shy * -1 / det;
		x: shx * y - (x * sy) / det;
		y: shy * x - (sx * y) / det;
		yourself! !

!AthensAffineTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/26/2011 02:09'!
loadIdentity
	"initialize with identity transform"
	sx := sy := 1.0.
	shx := shy := x := y := 0.0.
	
	! !

!AthensAffineTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/7/2012 14:01'!
restoreAfter: aBlock
	|previous|
	previous := self copy.
	aBlock ensure: [ 	self loadAffineTransform: previous ]! !

!AthensAffineTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/26/2011 01:46'!
rotateByDegrees: angle


	^ self rotateByRadians: angle degreesToRadians! !

!AthensAffineTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/26/2011 03:21'!
rotateByRadians: angle

	"multiply receiver by rotation matrix

| sx   shx   x   |        | cos -sin 0  |             | (sx*cos)+(shx*sin)   (-sx*sin+shx*cos)  x |
| shy sy     y   |   *   | sin   cos 0 |   ===> | (shy*cos)+(sy*sin)   (-shy*sin)+sy*cos) y |
|  0     0     1   |       | 0     0   1   |             |        0                                 0                1  |

"
	| cos sin newSx newSy |
	
	cos := angle cos.
	sin := angle sin.
	
	
	newSx := sx*cos + (shx*sin).
	newSy := sy*cos - (shy*sin).
	
	shx := shx*cos - (sx*sin).
	shy := shy*cos + (sy*sin).

	sx := newSx.
	sy := newSy.! !

!AthensAffineTransform methodsFor: 'accessing' stamp: ''!
scale
	^ sx@sy! !

!AthensAffineTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/21/2012 18:26'!
scaleBy: factor

	"multiply receiver by uniform scale matrix

| sx   shx   x   |       | (f x) 0 0  |             | sx*(f x)    shx*(f y)  x |
| shy sy     y   |   *   | 0 (f y) 0  |    ===> | shy*(f x)     sy*(f y)    y |
|  0     0     1  |        | 0 0     1  |                |  0      0              1  |

"	
	factor isPoint ifTrue: [
		sx := sx*factor x.
		shx := shx*factor y.
		sy := sy*factor y.
		shy := shy*factor x. 		
	] ifFalse: [
		sx := sx*factor.
		shx := shx*factor.
		sy := sy*factor.
		shy := shy*factor. ]
! !

!AthensAffineTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/26/2011 01:52'!
scaleX: fx Y: fy

	"multiply receiver by scale matrix

| sx   shx   x   |        | fx 0 0  |              | sx*fx    shx*fx  x |
| shy sy     y   |   *   | 0 fy 0  |    ===> | shy*fy  sy*fy   y |
|  0     0     1   |       | 0 0  1  |              |  0          0        1  |

"
	sx := sx*fx.
	shx := shx*fx.
	sy := sy*fy.
	shy := shy*fy.
! !

!AthensAffineTransform methodsFor: 'accessing' stamp: 'IgorStasenko 10/20/2011 18:48'!
shx
	^ shx! !

!AthensAffineTransform methodsFor: 'accessing' stamp: 'IgorStasenko 4/24/2012 14:04'!
shx: number
	shx := number! !

!AthensAffineTransform methodsFor: 'accessing' stamp: 'IgorStasenko 10/20/2011 18:48'!
shy
	^ shy! !

!AthensAffineTransform methodsFor: 'accessing' stamp: 'IgorStasenko 4/24/2012 14:04'!
shy: number
	shy := number! !

!AthensAffineTransform methodsFor: 'accessing' stamp: 'IgorStasenko 10/20/2011 18:48'!
sx
	^ sx! !

!AthensAffineTransform methodsFor: 'accessing' stamp: 'IgorStasenko 4/24/2012 14:04'!
sx: number
	sx := number! !

!AthensAffineTransform methodsFor: 'accessing' stamp: 'IgorStasenko 10/20/2011 18:48'!
sy
	^ sy! !

!AthensAffineTransform methodsFor: 'accessing' stamp: 'IgorStasenko 4/24/2012 14:04'!
sy: number
	sy := number! !

!AthensAffineTransform methodsFor: 'vector-transform' stamp: 'IgorStasenko 3/26/2011 03:02'!
transform: aPoint 
	| px py |
	
	px := aPoint x.
	py := aPoint y.
	^ Point 
		x: (sx*px +(shx*py) + x)
		y: (shy*px + (sy*py) + y) ! !

!AthensAffineTransform methodsFor: 'vector-transform' stamp: 'IgorStasenko 3/27/2011 18:40'!
transformX: px Y: py
	" transform x and y coordinates by receiver. Answer a Point"
	^ Point 
		x: (sx*px +(shx*py) + x)
		y: (shy*px + (sy*py) + y) ! !

!AthensAffineTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/26/2011 03:17'!
translateBy: aPoint

	^ self translateX: aPoint x Y: aPoint y! !

!AthensAffineTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/21/2012 18:23'!
translateX: px Y: py


" multiply receiver by translation matrix :

| sx   shx   x   |        | 1  0  px |             |	sx  shx  (sx*px + shx*py + x) |
| shy sy     y   |   *   | 0  1  py |   ===> | shy  sy  (shy*px + sy*py + y)  |
|  0     0     1   |       | 0  0   1  |             |  0     0    1  |

"
	x := (sx*px) + (shx*py) + x.
	y := (shy*px) + (sy*py) + y.
! !

!AthensAffineTransform methodsFor: 'transformations' stamp: 'IgorStasenko 10/24/2011 17:24'!
translateX: px y: py

" multiply receiver by translation matrix :

| sx   shx   x   |        | 1  0  px |             |	sx  shx  (sx*px + shx*py + x) |
| shy sy     y   |   *   | 0  1  py |   ===> | shy  sy  (shy*px + sy*py + y)  |
|  0     0     1   |       | 0  0   1  |             |  0     0    1  |

"
	x := (sx*px) + (shx*py) + x.
	y := (shy*px) + (sy*py) + y.
! !

!AthensAffineTransform methodsFor: 'accessing' stamp: ''!
translation
	^ x@y! !

!AthensAffineTransform methodsFor: 'accessing' stamp: ''!
translation: aPoint
	x := aPoint x.
	y := aPoint y.! !

!AthensAffineTransform methodsFor: 'transformations' stamp: 'IgorStasenko 10/25/2011 13:20'!
transposed
	
	| s |
	s := shx.
	shx := shy.
	shy := s.! !

!AthensAffineTransform methodsFor: 'accessing' stamp: 'IgorStasenko 10/20/2011 18:48'!
x
	^ x! !

!AthensAffineTransform methodsFor: 'accessing' stamp: 'IgorStasenko 4/24/2012 14:03'!
x: number
	x := number! !

!AthensAffineTransform methodsFor: 'accessing' stamp: 'IgorStasenko 10/20/2011 18:48'!
y
	^ y! !

!AthensAffineTransform methodsFor: 'accessing' stamp: 'IgorStasenko 4/24/2012 14:04'!
y: number
	y := number! !

AthensAffineTransform subclass: #AthensGenericTransform
	instanceVariableNames: 'w0 w1 w2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Matrices'!

!AthensGenericTransform commentStamp: '<historical>' prior: 0!
I am an extended version of my supeclass,
and I represent generic 3x3 matrix.

<Currently I have no use>!

!AthensGenericTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/26/2011 02:10'!
loadIdentity
	"initialize with identity transform"
	super loadIdentity.
	w0 := w1 := 0.0.
	w2 := 1.0! !

!AthensTransform methodsFor: 'accessing' stamp: 'IgorStasenko 8/31/2012 22:29'!
getMatrix
	"Answer an affine transformation matrix currently used by receiver.
	Note, that resulting matrix should be copied from receiver's data, i.e.
	no future modifications of receiver should affect the answered object.
	
	The default implementation is to answer a copy of receiver"
	
	^ self copy! !

!AthensTransform methodsFor: 'transformations' stamp: ''!
invert
	"i should invert"
	self subclassResponsibility! !

!AthensTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/7/2012 14:00'!
loadAffineTransform: anAthenTransform
	self subclassResponsibility! !

!AthensTransform methodsFor: 'transformations' stamp: 'IgorStasenko 12/20/2011 16:46'!
loadIdentity
	"load the identity matrix into receiver"
	self subclassResponsibility! !

!AthensTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/7/2012 13:52'!
restoreAfter: aBlock
	"i should save the current transform state, evaluate the block, and then restore the saved transform state"
	self subclassResponsibility! !

!AthensTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/26/2011 01:56'!
rotateByDegrees: angle

	self subclassResponsibility! !

!AthensTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/26/2011 01:55'!
rotateByRadians: angle

	self subclassResponsibility! !

!AthensTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/26/2011 01:55'!
scaleBy: factor

	self subclassResponsibility! !

!AthensTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/26/2011 01:55'!
scaleX: fx Y: fy

	self subclassResponsibility! !

!AthensTransform methodsFor: 'vector-transform' stamp: 'IgorStasenko 3/27/2011 18:42'!
transform: aPoint 
	self subclassResponsibility! !

!AthensTransform methodsFor: 'vector-transform' stamp: 'IgorStasenko 3/27/2011 18:42'!
transformX: px Y: py 
	self subclassResponsibility! !

!AthensTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/26/2011 01:55'!
translateBy: aPoint

	self subclassResponsibility! !

!AthensTransform methodsFor: 'transformations' stamp: 'IgorStasenko 12/21/2011 12:39'!
translateX: px Y: py

	self subclassResponsibility! !

!ImageFillStyle methodsFor: '*Athens-Core' stamp: 'IgorStasenko 10/12/2012 03:28'!
asAthensPaintOn: anAthensCanvas

	^ (anAthensCanvas cacheAt: self ifAbsentPut: [
		anAthensCanvas surface
		createFormPaint: form ])
		origin: origin;
		direction: (form width@0)
! !

!BitmapFillStyle methodsFor: '*Athens-Core' stamp: 'NicolaiHess 5/18/2015 09:14'!
asAthensPaintOn: anAthensCanvas
	| paint |
	paint := (anAthensCanvas surface createFormPaint: form)
		origin: origin;
		direction: self direction; yourself.
	self isTiled
		ifTrue: [ paint repeat ].
	^ paint! !

!Rectangle methodsFor: '*Athens-Core' stamp: 'IgorStasenko 5/6/2013 05:43'!
asAthensShapeOn: anAthensCanvas 
	"Just answer self. Receiver don't needs any conversion to be used with Athens"
	
	^ self! !

!Rectangle methodsFor: '*Athens-Core' stamp: 'IgorStasenko 3/27/2011 18:43'!
asPolygonTransformedBy: aTransform
	"answer a clock-wise oriented polygon, representing receiver,
	transformed by transform"
	
	^ {
		aTransform transform: origin.
		aTransform transformX: corner x Y: origin y.
		aTransform transform: corner.
		aTransform transformX: origin x Y: corner y.
	}! !

!Rectangle methodsFor: '*Athens-Core' stamp: 'IgorStasenko 8/30/2013 16:49'!
paintFillsUsing: aPaint on: anAthensCanvas 
	"This method is a part of rendering dispatch  Canvas->receiver->paint"
	
	^ aPaint athensFillRectangle: self on: anAthensCanvas! !

!SolidFillStyle methodsFor: '*Athens-Core' stamp: 'IgorStasenko 4/21/2011 15:01'!
asAthensPaintOn: anAthensCanvas 

	^ anAthensCanvas surface createSolidColorPaint: color! !

!Form methodsFor: '*Athens-Core' stamp: 'IgorStasenko 3/8/2012 13:45'!
asAthensPaintOn: canvas 

	^ canvas surface createFormPaint: self! !

!Color methodsFor: '*Athens-Core' stamp: 'IgorStasenko 8/30/2013 16:23'!
asAthensPaintOn: anAthensCanvas

	^ self
	
	"^ anAthensCanvas surface createSolidColorPaint: self"! !

!Color methodsFor: '*Athens-Core' stamp: 'IgorStasenko 9/3/2013 12:17'!
asStrokePaintOn: aCanvas 
	^ aCanvas surface createStrokePaintFor: self! !

!Color methodsFor: '*Athens-Core' stamp: 'IgorStasenko 9/3/2013 12:11'!
athensFillPath: aPath on: aCanvas 
	^ aCanvas surface fillPath: aPath withSolidColor:self! !

!Color methodsFor: '*Athens-Core' stamp: 'IgorStasenko 8/30/2013 16:49'!
athensFillRectangle: aRect on: anAthensCanvas

	^ anAthensCanvas surface fillRectangle: aRect withSolidColor:self! !

!Color methodsFor: '*Athens-Core' stamp: 'NicolaiHess 6/24/2014 22:50'!
strokePath: aPath on: aCanvas strokeWidth: width
	^ (aCanvas surface createSolidColorPaint: self) strokePath: aPath on: aCanvas strokeWidth: width! !

!Color methodsFor: '*Athens-Core' stamp: 'NicolaiHess 6/24/2014 22:53'!
strokeRect: aRect on: aCanvas strokeWidth: width
	^ (aCanvas surface createSolidColorPaint: self) strokeRect: aRect on: aCanvas strokeWidth: width! !

!GradientFillStyle methodsFor: '*Athens-Core' stamp: 'IgorStasenko 10/11/2012 16:16'!
asAthensPaintOn: anAthensCanvas

	^ AthensMorphicGradientPaint new gradient: self.
	"^ radial ifTrue: [
		anAthensCanvas surface
			createRadialGradient: colorRamp
			center:  (anAthensCanvas pathTransform inverseTransform: origin )
			radius: direction y ]
	ifFalse: [
		anAthensCanvas surface
			createLinearGradient: colorRamp
			origin: (anAthensCanvas pathTransform inverseTransform: origin)
			corner: (anAthensCanvas pathTransform inverseTransform: origin + direction ) ]
"! !

AthensCurveFlattener initialize!

AthensBezierConverter initialize!

AthensPathBuilder initialize!
